@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.35 2003/11/26 15:50:37 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.43 2003/12/05 11:28:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -72,6 +72,11 @@ static gboolean link_mouse_release_cb(BBSThreadView *view,
 				      OchushaBBSThread *thread,
 				      const gchar *link,
 				      OchushaApplication *application);
+static void write_response_cb(BBSThreadView *view,
+			      OchushaApplication *application);
+static void interactive_search_cb(BBSThreadView *view,
+				  BBSThreadViewSearchAction action,
+				  OchushaApplication *application);
 
 static void show_popup(OchushaApplication *application,
 		       OchushaBBSThread *thread,
@@ -98,6 +103,7 @@ static void start_element_cb(void *context, const char *name,
 static void end_element_cb(void *context, const char *name);
 static void characters_cb(void *context, const char *ch, int len);
 
+static pthread_mutex_t thread_info_lock;
 static pthread_mutex_t bbs_thread_ui_lock;
 
 /* ơС˥ᥤȤʤɤɽ뤿context ID */
@@ -106,6 +112,22 @@ static guint link_field_id;
 /* Ǹ˥ơСɽåФmessage ID */
 static guint message_id;
 
+
+#define THREAD_INFO_LOCK				\
+  if (pthread_mutex_lock(&thread_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't lock a mutex.\n");	\
+      abort();						\
+    }
+
+#define THREAD_INFO_UNLOCK				\
+  if (pthread_mutex_unlock(&thread_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't unlock a mutex.\n");	\
+      abort();						\
+    }
+
+
 #define BBS_THREAD_UI_LOCK				\
   if (pthread_mutex_lock(&bbs_thread_ui_lock) != 0)	\
     {							\
@@ -621,18 +643,23 @@ bbs_thread_info_free(BBSThreadGUIInfo *info)
 BBSThreadGUIInfo *
 ensure_bbs_thread_info(OchushaBBSThread *thread)
 {
-  BBSThreadGUIInfo *info;
-  BBS_THREAD_UI_LOCK
-    {
-      info = g_object_get_qdata(G_OBJECT(thread), bbs_thread_gui_info_id);
-      if (info == NULL)
-	{
-	  info = G_NEW0(BBSThreadGUIInfo, 1);
-	  g_object_set_qdata_full(G_OBJECT(thread), bbs_thread_gui_info_id,
-				  info, (GDestroyNotify)bbs_thread_info_free);
-	}
-    }
-  BBS_THREAD_UI_UNLOCK;
+  BBSThreadGUIInfo *info = g_object_get_qdata(G_OBJECT(thread),
+					      bbs_thread_gui_info_id);
+  if (info != NULL)
+    return info;
+
+  THREAD_INFO_LOCK
+  {
+    info = g_object_get_qdata(G_OBJECT(thread), bbs_thread_gui_info_id);
+    if (info == NULL)
+      {
+	info = G_NEW0(BBSThreadGUIInfo, 1);
+	g_object_set_qdata_full(G_OBJECT(thread), bbs_thread_gui_info_id,
+				info, (GDestroyNotify)bbs_thread_info_free);
+      }
+  }
+  THREAD_INFO_UNLOCK;
+
   return info;
 }
 
@@ -664,69 +691,73 @@ open_bbs_thread(OchushaApplication *application, OchushaBBSThread *thread,
     }
 
   BBS_THREAD_UI_LOCK
-    {
-      WorkerJob *job;
-      BBSThreadJobArgs *job_args;
-      OchushaAsyncBuffer *buffer;
+  {
+    WorkerJob *job;
+    BBSThreadJobArgs *job_args;
+    OchushaAsyncBuffer *buffer;
 
-      thread_view = bbs_thread_view_new(thread);
-      /*      gtk_widget_show(thread_view); */
+    thread_view = bbs_thread_view_new(thread);
+    /*      gtk_widget_show(thread_view); */
 
-      job = G_NEW0(WorkerJob, 1);
-      job_args = G_NEW0(BBSThreadJobArgs, 1);
-      buffer = ochusha_bbs_thread_get_responses_source(thread,
+    job = G_NEW0(WorkerJob, 1);
+    job_args = G_NEW0(BBSThreadJobArgs, 1);
+    buffer = ochusha_bbs_thread_get_responses_source(thread,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATE);
 
-      if (buffer == NULL)
-	{
-	  fprintf(stderr, "Couldn't access network!\n");
-	  G_FREE(job_args);
-	  G_FREE(job);
-	  gtk_object_sink(GTK_OBJECT(thread_view));
+    if (buffer == NULL)
+      {
+	fprintf(stderr, "Couldn't access network!\n");
+	G_FREE(job_args);
+	G_FREE(job);
+	gtk_object_sink(GTK_OBJECT(thread_view));
 
-	  BBS_THREAD_UI_UNLOCK;
-	  return NULL;
-	}
+	BBS_THREAD_UI_UNLOCK;
+	return NULL;
+      }
 
-      setup_for_tab_label_animation(buffer, tab_label);
+    setup_for_tab_label_animation(buffer, tab_label);
 
-      info->response_source_buffer = buffer;
-      info->rendering_done = FALSE;
+    info->response_source_buffer = buffer;
+    info->rendering_done = FALSE;
 
-      job_args->application = application;
-      job_args->thread = thread;
-      job_args->view = BBS_THREAD_VIEW(thread_view);
-      job_args->buffer = buffer;
-      job_args->start = 0;
-      job_args->number_of_responses = -1;
-      job_args->refresh = FALSE;
-      job_args->tab_label = tab_label;
+    job_args->application = application;
+    job_args->thread = thread;
+    job_args->view = BBS_THREAD_VIEW(thread_view);
+    job_args->buffer = buffer;
+    job_args->start = 0;
+    job_args->number_of_responses = -1;
+    job_args->refresh = FALSE;
+    job_args->tab_label = tab_label;
 
-      job->canceled = FALSE;
-      job->job = render_bbs_thread;
-      job->args = job_args;
+    job->canceled = FALSE;
+    job->job = render_bbs_thread;
+    job->args = job_args;
 
-      g_signal_connect(G_OBJECT(thread_view), "link_mouse_over",
-		       G_CALLBACK(link_mouse_over_cb), application);
-      g_signal_connect(G_OBJECT(thread_view), "link_mouse_out",
-		       G_CALLBACK(link_mouse_out_cb), application);
-      g_signal_connect(G_OBJECT(thread_view), "link_mouse_press",
-		       G_CALLBACK(link_mouse_press_cb), application);
-      g_signal_connect(G_OBJECT(thread_view), "link_mouse_release",
-		       G_CALLBACK(link_mouse_release_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "link_mouse_over",
+		     G_CALLBACK(link_mouse_over_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "link_mouse_out",
+		     G_CALLBACK(link_mouse_out_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "link_mouse_press",
+		     G_CALLBACK(link_mouse_press_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "link_mouse_release",
+		     G_CALLBACK(link_mouse_release_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "write_response",
+		     G_CALLBACK(write_response_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "interactive_search",
+		     G_CALLBACK(interactive_search_cb), application);
 
 #if DEBUG_ASYNC_BUFFER_MOST
-      fprintf(stderr, "open_bbs_thread: ref AsyncBuffer(%p)\n", buffer);
-      fprintf(stderr, "* before ref: buffer->ref_count=%d\n",
-	      G_OBJECT(buffer)->ref_count);
+    fprintf(stderr, "open_bbs_thread: ref AsyncBuffer(%p)\n", buffer);
+    fprintf(stderr, "* before ref: buffer->ref_count=%d\n",
+	    G_OBJECT(buffer)->ref_count);
 #endif
       /* öref_count䤹 */
-      g_object_ref(G_OBJECT(buffer));
-      g_object_ref(G_OBJECT(tab_label));
-      g_object_ref(G_OBJECT(thread_view));
-      commit_job(job);
-    }
+    g_object_ref(G_OBJECT(buffer));
+    g_object_ref(G_OBJECT(tab_label));
+    g_object_ref(G_OBJECT(thread_view));
+    commit_job(job);
+  }
   BBS_THREAD_UI_UNLOCK;
 
   return thread_view;
@@ -982,9 +1013,19 @@ link_mouse_release_cb(BBSThreadView *view, GdkEventButton *event,
 }
 
 
-#define DEFAULT_SUBJECT_COLOR	"red"
-#define DEFAULT_NAME_COLOR	"green"
-#define DEFAULT_LINK_COLOR	"blue"
+static void
+write_response_cb(BBSThreadView *view, OchushaApplication *application)
+{
+  write_response(application);
+}
+
+
+static void
+interactive_search_cb(BBSThreadView *view, BBSThreadViewSearchAction action,
+		      OchushaApplication *application)
+{
+  start_thread_search(application);
+}
 
 
 static gboolean
@@ -1075,17 +1116,22 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 #endif
 
   if (info->offsets == NULL)
-    info->offsets = G_NEW0(gint, MAX_RESPONSE);
+    {
+      info->offsets = G_NEW0(guint, MAX_RESPONSE);
+      info->offsets_length = MAX_RESPONSE;
+    }
   else 
-    memset(info->offsets, 0, MAX_RESPONSE * sizeof(gint));
+    {
+      memset(info->offsets, 0, info->offsets_length * sizeof(guint));
+    }
 
   if (context.handler.converter == (iconv_t)-1)
     {
       fprintf(stderr, "Out of memory\n");
       BBS_THREAD_UI_LOCK
-	{
-	  info->rendering_done = TRUE;
-	}
+      {
+	info->rendering_done = TRUE;
+      }
       BBS_THREAD_UI_UNLOCK;
       return;
     }
@@ -1107,10 +1153,23 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 
   if (job_args->refresh)
     {
-      /* ɥ쥹˰*/
+      int index = job_args->start - 2;
       if (info->last_read != NULL)
 	bbs_thread_view_delete_mark(job_args->view, info->last_read);
-      info->last_read = bbs_thread_view_create_mark(job_args->view);
+
+      /* ɥ쥹ΤˤϺǸδɥ쥹ľˤ˰դ롣*/
+      if (index > 0
+	  && info->offsets_length > index && info->offsets[index] != 0)
+	{
+	  bbs_thread_view_scroll_to_offset(job_args->view,
+					   info->offsets[index]);
+	  info->last_read
+	    = bbs_thread_view_create_mark_at_offset(job_args->view,
+						    info->offsets[index]);
+	}
+      else
+	info->last_read = bbs_thread_view_create_mark(job_args->view);
+
       info->next_mark = info->last_read;
 #if DEBUG_GUI
       fprintf(stderr, "create mark at the tail response.\n");
@@ -1152,9 +1211,9 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
   g_object_unref(G_OBJECT(buffer));
 
   BBS_THREAD_UI_LOCK
-    {
-      info->rendering_done = TRUE;
-    }
+  {
+    info->rendering_done = TRUE;
+  }
   BBS_THREAD_UI_UNLOCK;
 
 #if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
@@ -1234,6 +1293,14 @@ render_response(OchushaBBSThread *thread, gint number,
 
   if (context->last_read_number >= 0)
     {
+      if (number >= info->offsets_length)
+	{
+	  unsigned int new_len = info->offsets_length * 2;
+	  info->offsets = G_REALLOC(info->offsets, new_len);
+	  memset(info->offsets + info->offsets_length, 0,
+		 (new_len - info->offsets_length) * sizeof(guint));
+	  info->offsets_length = new_len;
+	}
       info->offsets[number] = bbs_thread_view_get_current_offset(view);
     }
 
@@ -1386,6 +1453,14 @@ render_broken_response(OchushaBBSThread *thread, gint number,
 
   if (context->last_read_number >= 0)
     {
+      if (number >= info->offsets_length)
+	{
+	  unsigned int new_len = info->offsets_length * 2;
+	  info->offsets = G_REALLOC(info->offsets, new_len);
+	  memset(info->offsets + info->offsets_length, 0,
+		 (new_len - info->offsets_length) * sizeof(guint));
+	  info->offsets_length = new_len;
+	}
       info->offsets[number] = bbs_thread_view_get_current_offset(view);
     }
 
@@ -1924,9 +1999,9 @@ popup_expose_event_cb(GtkWidget *widget, GdkEventExpose *event,
   if (requisition.width > 0 && need_popup_adjustment)
     {
       BBS_THREAD_UI_LOCK
-	{
-	  need_popup_adjustment = FALSE;
-	}
+      {
+	need_popup_adjustment = FALSE;
+      }
       BBS_THREAD_UI_UNLOCK;
 
       if (popup_context->renderer_context.application->enable_smart_popup_placement)
@@ -2254,9 +2329,9 @@ show_popup_real(PopupContext *popup_context)
       if (requisition.width == 0)
 	{
 	  BBS_THREAD_UI_LOCK
-	    {
-	      need_popup_adjustment = TRUE;
-	    }
+	  {
+	    need_popup_adjustment = TRUE;
+	  }
 	  BBS_THREAD_UI_UNLOCK;
 
 	  use_two_steps_hack = TRUE;
@@ -2272,9 +2347,9 @@ show_popup_real(PopupContext *popup_context)
 			    requisition.width + THREAD_VIEW_EXTRA_WIDTH,
 			    requisition.height);
 	  BBS_THREAD_UI_LOCK
-	    {
-	      need_popup_adjustment = FALSE;
-	    }
+	  {
+	    need_popup_adjustment = FALSE;
+	  }
 	  BBS_THREAD_UI_UNLOCK;
 	}
 
@@ -2364,7 +2439,7 @@ popup_close_delay_timeout(PopupContext *popup_context)
   if (!popup_context->now_pointed
       && !(popup_context->renderer_context.view != NULL
 	   && (child_context = g_object_get_qdata(G_OBJECT(popup_context->renderer_context.view), popup_context_id)) != NULL
-       && child_context->renderer_context.view != NULL))
+	   && child_context->renderer_context.view != NULL))
     {
 #if DEBUG_POPUP
       fprintf(stderr, "popup_close_delay_timeout: now_pointed=%d child=%p\n",
@@ -2404,6 +2479,11 @@ hide_popup(PopupContext *popup_context)
 	= g_timeout_add(application->popup_close_delay,
 			(GSourceFunc)popup_close_delay_timeout, popup_context);
     }
+  else
+    {
+      /* ɽԤΥݥåץåפɽΤ롣 */
+      hide_popup_real(popup_context);
+    }
 }
 
 
@@ -2420,74 +2500,74 @@ refresh_thread(OchushaApplication *application, GtkWidget *widget,
 #endif
 
   BBS_THREAD_UI_LOCK
-    {
-      OchushaAsyncBuffer *buffer;
-      OchushaAsyncBuffer *old_buffer;
+  {
+    OchushaAsyncBuffer *buffer;
+    OchushaAsyncBuffer *old_buffer;
 
-      if (!info->rendering_done)
-	{
+    if (!info->rendering_done)
+      {
 #if DEBUG_GUI_MOST
-	  fprintf(stderr, "refresh_thread: refresh has already been requested.\n");
+	fprintf(stderr, "refresh_thread: refresh has already been requested.\n");
 #endif
-	  goto finish_refresh;	/* 󥰤򤷤Ƥ */
-	}
-      job = G_NEW0(WorkerJob, 1);
-      job_args = G_NEW0(BBSThreadJobArgs, 1);
-      old_buffer = info->response_source_buffer;
-      buffer = ochusha_bbs_thread_get_responses_source(thread,
+	goto finish_refresh;	/* 󥰤򤷤Ƥ */
+      }
+    job = G_NEW0(WorkerJob, 1);
+    job_args = G_NEW0(BBSThreadJobArgs, 1);
+    old_buffer = info->response_source_buffer;
+    buffer = ochusha_bbs_thread_get_responses_source(thread,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_UPDATE);
 
-      if (buffer == NULL)
-	{
-	  fprintf(stderr, "Couldn't access network!\n");
-	  BBS_THREAD_UI_UNLOCK;
-	  if (job != NULL)
-	    G_FREE(job);
-	  if (job_args != NULL)
-	    G_FREE(job_args);
-	  return;
-	}
+    if (buffer == NULL)
+      {
+	fprintf(stderr, "Couldn't access network!\n");
+	BBS_THREAD_UI_UNLOCK;
+	if (job != NULL)
+	  G_FREE(job);
+	if (job_args != NULL)
+	  G_FREE(job_args);
+	return;
+      }
 
-      setup_for_tab_label_animation(buffer, tab_label);
+    setup_for_tab_label_animation(buffer, tab_label);
 
-      info->response_source_buffer = buffer;
-      info->rendering_done = FALSE;
+    info->response_source_buffer = buffer;
+    info->rendering_done = FALSE;
 
-      job_args->application = application;
-      job_args->thread = thread;
-      job_args->view = view;
-      job_args->buffer = buffer;
-      job_args->start = thread->number_of_responses_read;
-      job_args->number_of_responses = -1;
-      job_args->refresh = TRUE;
-      job_args->tab_label = tab_label;
+    job_args->application = application;
+    job_args->thread = thread;
+    job_args->view = view;
+    job_args->buffer = buffer;
+    job_args->start = thread->number_of_responses_read;
+    job_args->number_of_responses = -1;
+    job_args->refresh = TRUE;
+    job_args->tab_label = tab_label;
 
-      job->canceled = FALSE;
-      job->job = render_bbs_thread;
-      job->args = job_args;
+    job->canceled = FALSE;
+    job->job = render_bbs_thread;
+    job->args = job_args;
 
 #if DEBUG_ASYNC_BUFFER_MOST
-      fprintf(stderr, "refresh_thread: ref AsyncBuffer(%p)\n", buffer);
-      fprintf(stderr, "* before unref: buffer->ref_count=%d\n",
-	      G_OBJECT(buffer)->ref_count);
+    fprintf(stderr, "refresh_thread: ref AsyncBuffer(%p)\n", buffer);
+    fprintf(stderr, "* before unref: buffer->ref_count=%d\n",
+	    G_OBJECT(buffer)->ref_count);
 #endif
-      g_object_ref(G_OBJECT(buffer));
-      g_object_ref(G_OBJECT(tab_label));
-      g_object_ref(G_OBJECT(view));
-      commit_job(job);
+    g_object_ref(G_OBJECT(buffer));
+    g_object_ref(G_OBJECT(tab_label));
+    g_object_ref(G_OBJECT(view));
+    commit_job(job);
 
-      if (old_buffer != NULL)
-	{
+    if (old_buffer != NULL)
+      {
 #if DEBUG_ASYNC_BUFFER_MOST
-	  fprintf(stderr, "refresh_thread: unref AsyncBuffer(%p)\n",
-		  old_buffer);
-	  fprintf(stderr, "* before unref: old_buffer->ref_count=%d\n",
-		  G_OBJECT(old_buffer)->ref_count);
+	fprintf(stderr, "refresh_thread: unref AsyncBuffer(%p)\n",
+		old_buffer);
+	fprintf(stderr, "* before unref: old_buffer->ref_count=%d\n",
+		G_OBJECT(old_buffer)->ref_count);
 #endif
-	  g_object_unref(G_OBJECT(old_buffer));
-	}
-    }
+	g_object_unref(G_OBJECT(old_buffer));
+      }
+  }
 
  finish_refresh:
   BBS_THREAD_UI_UNLOCK;
