@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.h,v 1.10 2003/11/25 18:19:23 fuyu Exp $
+ * $Id: bbs_thread_ui.h,v 1.11 2003/11/29 08:54:28 fuyu Exp $
  */
 
 #ifndef _BBSTHREAD_UI_H_
@@ -70,7 +70,8 @@ typedef struct _BBSThreadGUIInfo
   GtkTextMark *next_mark;
 
   /* ġΥ쥹GtkTextBuffer⥪եå */
-  gint *offsets;
+  guint *offsets;
+  guint offsets_length;
 
   gboolean rendering_done;
 
