@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.19 2003/11/25 18:19:23 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.23 2003/12/02 02:28:13 fuyu Exp $
  */
 
 #include "config.h"
@@ -141,6 +141,9 @@ enum {
   VIEW_ENTER_NOTIFY_SIGNAL,
   VIEW_LEAVE_NOTIFY_SIGNAL,
   SCROLL_TO_SIGNAL,
+  INTERACTIVE_SEARCH_SIGNAL,
+  WRITE_RESPONSE_SIGNAL,
+  CLOSE_THREAD_SIGNAL,
   LAST_SIGNAL
 };
 
@@ -152,7 +155,8 @@ enum {
 
 
 static GtkTextViewClass *parent_class = NULL;
-static gint bbs_thread_view_signals[LAST_SIGNAL] = { 0, 0, 0, 0, 0, 0, 0 };
+static gint bbs_thread_view_signals[LAST_SIGNAL] =
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -208,14 +212,10 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   default_tag_table = gtk_text_tag_table_new();
   klass->tag_table = default_tag_table;
 
-
+  /* TODO: طѹǽˤ */
   tag = gtk_text_tag_new("thread_title");
   g_object_set(G_OBJECT(tag),
 	       "foreground", "red",
-#if 0
-	       "weight", PANGO_WEIGHT_BOLD,
-	       "scale", PANGO_SCALE_X_LARGE,
-#endif
 	       NULL);
   gtk_text_tag_table_add(default_tag_table, tag);
 
@@ -372,6 +372,34 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
 
+  bbs_thread_view_signals[INTERACTIVE_SEARCH_SIGNAL] =
+    g_signal_new("interactive_search",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, interactive_search),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__INT,
+		 G_TYPE_NONE, 1,
+		 G_TYPE_INT);
+
+  bbs_thread_view_signals[WRITE_RESPONSE_SIGNAL] =
+    g_signal_new("write_response",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, write_response),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+
+  bbs_thread_view_signals[CLOSE_THREAD_SIGNAL] =
+    g_signal_new("close_thread",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, close_thread),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+
   gtk_binding_entry_add_signal(binding_set, GDK_Home, GDK_CONTROL_MASK,
 			       "scroll_to", 1,
 			       G_TYPE_INT, SCROLL_TO_BUFFER_START);
@@ -396,6 +424,21 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 			       G_TYPE_INT, -1,
 			       G_TYPE_BOOLEAN, FALSE);
 
+  gtk_binding_entry_add_signal(binding_set, GDK_S, GDK_CONTROL_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       BBS_THREAD_VIEW_SEARCH_ACTION_FORWARD);
+  gtk_binding_entry_add_signal(binding_set, GDK_R, GDK_CONTROL_MASK,
+			       "interactive_search", 1,
+			       G_TYPE_INT,
+			       BBS_THREAD_VIEW_SEARCH_ACTION_BACKWARD);
+
+  gtk_binding_entry_add_signal(binding_set, GDK_A, 0,
+			       "write_response", 0);
+
+  gtk_binding_entry_add_signal(binding_set, GDK_W, GDK_CONTROL_MASK,
+			       "close_thread", 0);
+
   klass->link_mouse_over = NULL;
   klass->link_mouse_out = NULL;
   klass->link_mouse_press = NULL;
@@ -813,7 +856,7 @@ button_release_event_cb(GtkTextView *text_view, GdkEventButton *event,
 		    bbs_thread_view->thread,
 		    link,
 		    &handled);
-      return TRUE;
+/*      return TRUE; */
     }
 
   return FALSE;
@@ -1217,6 +1260,33 @@ bbs_thread_view_scroll_to_mark(BBSThreadView *view, GtkTextMark *mark)
 }
 
 
+GtkTextMark *
+bbs_thread_view_create_mark_at_offset(BBSThreadView *view, gint offset)
+{
+  GtkTextIter iter;
+  GtkTextMark *mark;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL,
+		       NULL);
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
+  mark = gtk_text_buffer_create_mark(view->text_buffer, NULL, &iter, TRUE); 
+  return mark;
+}
+
+
+void
+bbs_thread_view_place_cursor_at_offset(BBSThreadView *view, gint offset)
+{
+  GtkTextIter iter;
+
+  g_return_if_fail(IS_BBS_THREAD_VIEW(view) && view->text_buffer != NULL);
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
+  gtk_text_buffer_place_cursor(view->text_buffer, &iter);
+}
+
+
 /* Ϣ */
 static void
 bbs_thread_view_scroll_to_offsets(BBSThreadView *view,
