@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.30 2003/11/26 04:13:26 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.34 2003/12/04 07:32:17 fuyu Exp $
  */
 
 #include "config.h"
@@ -159,9 +159,25 @@ static GtkWidget *create_new_thread_dialog(OchushaApplication *application,
 /* Υå˰٤ǤⳫ(OchushaBulletinBoard *)Υꥹȡ*/
 static GSList *board_list;
 
+static pthread_mutex_t category_info_lock;
 static pthread_mutex_t board_list_lock;
 
 
+#define CATEGORY_INFO_LOCK				\
+  if (pthread_mutex_lock(&category_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't lock a mutex.\n");	\
+      abort();						\
+    }
+
+#define CATEGORY_INFO_UNLOCK				\
+  if (pthread_mutex_unlock(&category_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't unlock a mutex.\n");	\
+      abort();						\
+    }
+
+
 #define BOARD_LIST_LOCK					\
   if (pthread_mutex_lock(&board_list_lock) != 0)	\
     {							\
@@ -258,6 +274,9 @@ initialize_boardlist(OchushaApplication *application)
 		   G_CALLBACK(gtk_widget_destroyed),
 		   &application->boardlist_view);
 #endif
+  boardlist_view_set_default_open_in_tab(
+				BOARDLIST_VIEW(application->boardlist_view),
+				application->default_open_in_tab);
   gtk_widget_show(application->boardlist_view);
 
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
@@ -394,14 +413,19 @@ trace_free(gpointer pointer)
 static BoardCategoryGUIInfo *
 ensure_board_category_info(OchushaBoardCategory *category)
 {
-  BoardCategoryGUIInfo *info = g_object_get_qdata(G_OBJECT(category),
-						  category_gui_info_id);
-  if (info == NULL)
-    {
-      info = G_NEW0(BoardCategoryGUIInfo, 1);
-      g_object_set_qdata_full(G_OBJECT(category), category_gui_info_id,
-			      info, (GDestroyNotify)TRACE_FREE);
-    }
+  BoardCategoryGUIInfo *info;
+
+  CATEGORY_INFO_LOCK
+  {
+    info = g_object_get_qdata(G_OBJECT(category), category_gui_info_id);
+    if (info == NULL)
+      {
+	info = G_NEW0(BoardCategoryGUIInfo, 1);
+	g_object_set_qdata_full(G_OBJECT(category), category_gui_info_id,
+				info, (GDestroyNotify)TRACE_FREE);
+      }
+  }
+  CATEGORY_INFO_UNLOCK;
 
   return info;
 }
@@ -951,10 +975,10 @@ item_view_required_cb(PanedNotebook *paned_notebook, gpointer item,
     }
 
   BOARD_LIST_LOCK
-    {
-      if (g_slist_find(board_list, board) == NULL)
-	board_list = g_slist_append(board_list, board);
-    }
+  {
+    if (g_slist_find(board_list, board) == NULL)
+      board_list = g_slist_append(board_list, board);
+  }
   BOARD_LIST_UNLOCK;
 
   gtk_widget_show(*item_view);
@@ -963,14 +987,14 @@ item_view_required_cb(PanedNotebook *paned_notebook, gpointer item,
 
 
 static void
-item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board,
+item_view_being_closed_cb(PanedNotebook *paned_notebook,
+			  OchushaBulletinBoard *board,
 			  GtkWidget *item_view,
 			  OchushaApplication *application)
 {
-  OchushaAsyncBuffer *buffer = snatch_threadlist_source_buffer_for_board(
-							board, application);
+  OchushaAsyncBuffer *buffer
+    = snatch_threadlist_source_buffer_for_board(board, application);
 
-#ifndef SUSPEND_ALL_FOR_DESTRUCTION	/* ¸ */
   /*
    * MEMO: ɽƤ륦åȤºݤ˼ΤƤ륿ߥ
    *       򤺤餹
@@ -999,30 +1023,6 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *b
 
       gdk_threads_enter();
     }
-#else
-  /*
-   * ΥɽĤ褦ȤƤΤǡϢthreadߤᡢ
-   * ¾GUIϢthread򥵥ڥɤίäƤ륤٥Ȥ
-   * Ƥ롣ǽʸ¤᤯٥Ȥò뤿ᡢƤthread
-   * ڥɤˡ˴Ϣthread϶Ūߤ롣
-   *
-   * ġĤ褦ˤ顢ब򤱤褦ʵΤġġ
-   */
-
-  gdk_threads_leave();
-
-  ochusha_async_buffer_suspend_all();
-
-  if (buffer != NULL)
-    ochusha_async_buffer_terminate(buffer);
-
-  gdk_threads_enter();
-
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
-  ochusha_async_buffer_resume_all();
-#endif
 
   if (buffer != NULL)
     g_object_unref(G_OBJECT(buffer));
@@ -1045,9 +1045,7 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBulletinBoard *board,
     }
 
   finalize_bulletin_board_view(board, item_view);
-#ifndef SUSPEND_ALL_FOR_DESTRUCTION
   delayed_g_object_unref(G_OBJECT(item_view));
-#endif
 }
 
 
@@ -1085,6 +1083,7 @@ page_switched_cb(PanedNotebook *paned_notebook,
       OchushaBulletinBoard *board
 	= (OchushaBulletinBoard *)paned_notebook_get_item(paned_notebook, new_page);
       BulletinBoardGUIInfo *info;
+      GtkWidget *tmp_widget;
 
       g_return_if_fail(board != NULL);
       info = ensure_bulletin_board_info(board, application);
@@ -1096,6 +1095,10 @@ page_switched_cb(PanedNotebook *paned_notebook,
 			     application->threadlist_height);
       gtk_toggle_button_set_active(application->filter_button,
 				   info->enable_filter);
+      tmp_widget = paned_notebook_get_selector(PANED_NOTEBOOK(new_page));
+      tmp_widget = gtk_bin_get_child(GTK_BIN(tmp_widget));
+      threadlist_view_set_default_open_in_tab(THREADLIST_VIEW(tmp_widget),
+					application->default_open_in_tab);
     }
 }
 
