@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.54 2003/11/27 00:39:10 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.65 2003/12/05 12:42:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -66,6 +66,10 @@ static void advance_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			    PanedNotebook *paned_notebook);
 static void back_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			 PanedNotebook *paned_notebook);
+static void write_response_cb(ThreadlistView *view, OchushaBBSThread *thread,
+			      OchushaApplication *application);
+static void close_threadlist_cb(ThreadlistView *view,
+			      OchushaApplication *application);
 static void toggle_mark_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			   OchushaApplication *application);
 static void toggle_hide_cb(ThreadlistView *view, OchushaBBSThread *thread,
@@ -122,9 +126,23 @@ static GtkWidget *create_write_dialog(OchushaApplication *application,
 				      OchushaBBSThread *thread);
 
 
+static pthread_mutex_t board_info_lock;
 static pthread_mutex_t thread_list_lock;
 static pthread_cond_t thread_list_condition;
 
+#define BOARD_INFO_LOCK					\
+  if (pthread_mutex_lock(&board_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't lock a mutex.\n");	\
+      abort();						\
+    }
+
+#define BOARD_INFO_UNLOCK				\
+  if (pthread_mutex_unlock(&board_info_lock) != 0)	\
+    {							\
+      fprintf(stderr, "Couldn't unlock a mutex.\n");	\
+      abort();						\
+    }
 
 #define THREAD_LIST_LOCK				\
   if (pthread_mutex_lock(&thread_list_lock) != 0)	\
@@ -171,6 +189,12 @@ static GQuark application_id;
 void
 prepare_board_ui_initialization(OchushaApplication *application)
 {
+ if (pthread_mutex_init(&board_info_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+
  if (pthread_mutex_init(&thread_list_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
@@ -327,24 +351,22 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board,
 			   OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info;
-  THREAD_LIST_LOCK
-    {
-      info = g_object_get_qdata(G_OBJECT(board), board_info_id);
-      if (info == NULL)
-	{
-	  info = G_NEW0(BulletinBoardGUIInfo, 1);
-	  g_object_set_qdata_full(G_OBJECT(board), board_info_id,
-				info,
+  BOARD_INFO_LOCK
+  {
+    info = g_object_get_qdata(G_OBJECT(board), board_info_id);
+    if (info == NULL)
+      {
+	info = G_NEW0(BulletinBoardGUIInfo, 1);
+	g_object_set_qdata_full(G_OBJECT(board), board_info_id, info,
 				(GDestroyNotify)bulletin_board_gui_info_free);
-	  g_signal_connect(G_OBJECT(board), "threadlist_read_thread_element",
-			   G_CALLBACK(threadlist_read_thread_element_cb),
-			   NULL);
-	  g_signal_connect(G_OBJECT(board), "threadlist_write_thread_element",
-			   G_CALLBACK(threadlist_write_thread_element_cb),
-			   application);
-	}
-    }
-  THREAD_LIST_UNLOCK;
+	g_signal_connect(G_OBJECT(board), "threadlist_read_thread_element",
+			 G_CALLBACK(threadlist_read_thread_element_cb), NULL);
+	g_signal_connect(G_OBJECT(board), "threadlist_write_thread_element",
+			 G_CALLBACK(threadlist_write_thread_element_cb),
+			 application);
+      }
+  }
+  BOARD_INFO_UNLOCK;
 
   return info;
 }
@@ -534,170 +556,180 @@ open_bulletin_board(OchushaApplication *application,
     }
 
   THREAD_LIST_LOCK
-    {
-      WorkerJob *job;
-      ThreadlistJobArgs *job_args;
-      GtkWidget *scrolled_window;
-      GtkWidget *view;
-      GtkToolbar *toolbar;
+  {
+    WorkerJob *job;
+    ThreadlistJobArgs *job_args;
+    GtkWidget *scrolled_window;
+    GtkWidget *view;
+    GtkToolbar *toolbar;
 
-      if (info->is_busy)
-	{
+    if (info->is_busy)
+      {
 #if DEBUG_THREAD
-	  fprintf(stderr, "board is busy.\n");
+	fprintf(stderr, "board is busy.\n");
 #endif
-	  THREAD_LIST_UNLOCK;
-	  return NULL;
-	}
+	THREAD_LIST_UNLOCK;
+	return NULL;
+      }
 
-      scrolled_window = gtk_scrolled_window_new(NULL, NULL);
-      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
-				     GTK_POLICY_AUTOMATIC,
-				     GTK_POLICY_AUTOMATIC);
-      gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
-					  GTK_SHADOW_ETCHED_IN);
-      view = threadlist_view_new();
-      g_object_set_qdata(G_OBJECT(view), application_id, application);
-      customize_threadlist_view_contents(application, THREADLIST_VIEW(view));
+    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+				   GTK_POLICY_AUTOMATIC,
+				   GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolled_window),
+					GTK_SHADOW_ETCHED_IN);
+    view = threadlist_view_new();
+    g_object_set_qdata(G_OBJECT(view), application_id, application);
+    customize_threadlist_view_contents(application, THREADLIST_VIEW(view));
+    threadlist_view_set_default_open_in_tab(THREADLIST_VIEW(view),
+					    application->default_open_in_tab);
 
-      job = G_NEW0(WorkerJob, 1);
-      job_args = G_NEW0(ThreadlistJobArgs, 1);
+    job = G_NEW0(WorkerJob, 1);
+    job_args = G_NEW0(ThreadlistJobArgs, 1);
 
-      gtk_widget_show(view);
-      gtk_container_add(GTK_CONTAINER(scrolled_window), view);
-      gtk_widget_show(scrolled_window);
+    gtk_widget_show(view);
+    gtk_container_add(GTK_CONTAINER(scrolled_window), view);
+    gtk_widget_show(scrolled_window);
 
-      paned_notebook
-	= paned_notebook_new_with_selector(application->threadlist_pane_style,
-					   scrolled_window);
-      g_signal_connect(G_OBJECT(view), "open_thread_view",
-		       G_CALLBACK(open_thread_view_cb), paned_notebook);
-      gtk_paned_set_position(GTK_PANED(paned_notebook),
-			     application->threadlist_height);
-      paned_notebook_set_tab_shrinkable(PANED_NOTEBOOK(paned_notebook),
-					application->thread_tab_shrinkable);
-      paned_notebook_set_minimum_tab_label_size(PANED_NOTEBOOK(paned_notebook),
+    paned_notebook
+      = paned_notebook_new_with_selector(application->threadlist_pane_style,
+					 scrolled_window);
+    g_signal_connect(G_OBJECT(view), "open_thread_view",
+		     G_CALLBACK(open_thread_view_cb), paned_notebook);
+    gtk_paned_set_position(GTK_PANED(paned_notebook),
+			   application->threadlist_height);
+    paned_notebook_set_tab_shrinkable(PANED_NOTEBOOK(paned_notebook),
+				      application->thread_tab_shrinkable);
+    paned_notebook_set_minimum_tab_label_size(PANED_NOTEBOOK(paned_notebook),
 					application->thread_tab_minimum_size);
-      paned_notebook_set_tooltips(PANED_NOTEBOOK(paned_notebook),
-				  application->thread_tab_enable_tooltips);
+    paned_notebook_set_tooltips(PANED_NOTEBOOK(paned_notebook),
+				application->thread_tab_enable_tooltips);
+    paned_notebook_set_tab_policy(PANED_NOTEBOOK(paned_notebook),
+				  application->thread_tab_always_show
+				  ? GTK_POLICY_ALWAYS
+				  : GTK_POLICY_AUTOMATIC);
 
-      toolbar = paned_notebook_get_toolbar(PANED_NOTEBOOK(paned_notebook));
+    toolbar = paned_notebook_get_toolbar(PANED_NOTEBOOK(paned_notebook));
 
-      gtk_toolbar_prepend_space(toolbar);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_NEW,
-			       _("Write a response to the current thread"),
-			       "write_response_to_current_thread",
-			       GTK_SIGNAL_FUNC(write_response_button_cb),
-			       application,
-			       0);
+    gtk_toolbar_prepend_space(toolbar);
+    gtk_toolbar_insert_stock(toolbar, OCHUSHA_STOCK_WRITE_RESPONSE,
+			     _("Write a response to the current thread"),
+			     "write_response_to_current_thread",
+			     GTK_SIGNAL_FUNC(write_response_button_cb),
+			     application,
+			     0);
 
-      gtk_toolbar_prepend_space(toolbar);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_SELECT_FONT,
-			       _("Select font to render threads"),
-			       "select_font_to_render_threads",
-			       GTK_SIGNAL_FUNC(select_font_button_cb),
-			       application,
-			       0);
+    gtk_toolbar_prepend_space(toolbar);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_SELECT_FONT,
+			     _("Select font to render threads"),
+			     "select_font_to_render_threads",
+			     GTK_SIGNAL_FUNC(select_font_button_cb),
+			     application,
+			     0);
 
-      gtk_toolbar_prepend_space(toolbar);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
-			       _("Refresh current thread"),
-			       "refresh_current_thread",
-			       GTK_SIGNAL_FUNC(refresh_thread_button_cb),
-			       application,
-			       0);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
-			       _("Start text search on current thread"),
-			       "start_text_search_on_thread",
-			       GTK_SIGNAL_FUNC(start_thread_search_button_cb),
-			       application,
-			       0);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_JUMP_TO,
-			       _("Jump to new comer response"),
-			       "jump_to_new_comer_response",
-			       GTK_SIGNAL_FUNC(jump_to_new_comer_button_cb),
-			       application,
-			       0);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_BOTTOM,
-			       _("Go to bottom of the response view"),
-			       "go_to_bottom_of_response",
-			       GTK_SIGNAL_FUNC(go_to_bottom_button_cb),
-			       application,
-			       0);
-      gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_TOP,
-			       _("Go to top of the response view"),
-			       "go_to_top_of_response",
-			       GTK_SIGNAL_FUNC(go_to_top_button_cb),
-			       application,
-			       0);
+    gtk_toolbar_prepend_space(toolbar);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_REFRESH,
+			     _("Refresh current thread"),
+			     "refresh_current_thread",
+			     GTK_SIGNAL_FUNC(refresh_thread_button_cb),
+			     application,
+			     0);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_FIND,
+			     _("Start text search on current thread"),
+			     "start_text_search_on_thread",
+			     GTK_SIGNAL_FUNC(start_thread_search_button_cb),
+			     application,
+			     0);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_JUMP_TO,
+			     _("Jump to new comer response"),
+			     "jump_to_new_comer_response",
+			     GTK_SIGNAL_FUNC(jump_to_new_comer_button_cb),
+			     application,
+			     0);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_BOTTOM,
+			     _("Go to bottom of the response view"),
+			     "go_to_bottom_of_response",
+			     GTK_SIGNAL_FUNC(go_to_bottom_button_cb),
+			     application,
+			     0);
+    gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_TOP,
+			     _("Go to top of the response view"),
+			     "go_to_top_of_response",
+			     GTK_SIGNAL_FUNC(go_to_top_button_cb),
+			     application,
+			     0);
 
-      gtk_widget_show(paned_notebook);
+    gtk_widget_show(paned_notebook);
 
-      g_signal_connect(G_OBJECT(view), "advance_view",
-		       G_CALLBACK(advance_view_cb), paned_notebook);
-      g_signal_connect(G_OBJECT(view), "back_view",
-		       G_CALLBACK(back_view_cb), paned_notebook);
-      g_signal_connect(G_OBJECT(view), "toggle_mark",
-		       G_CALLBACK(toggle_mark_cb), application);
-      g_signal_connect(G_OBJECT(view), "toggle_hide",
-		       G_CALLBACK(toggle_hide_cb), application);
-      g_signal_connect(G_OBJECT(view), "reset_thread",
-		       G_CALLBACK(reset_thread_cb), application);
-      g_signal_connect(G_OBJECT(view), "copy_thread_url",
-		       G_CALLBACK(copy_thread_url_cb), application);
-      g_signal_connect(G_OBJECT(view), "mark_thread",
-		       G_CALLBACK(mark_thread_cb), application);
-      g_signal_connect(G_OBJECT(view), "thread_mouse_over",
-		       G_CALLBACK(thread_mouse_over_cb), application);
-      g_signal_connect(G_OBJECT(view), "thread_mouse_out",
-		       G_CALLBACK(thread_mouse_out_cb), application);
+    g_signal_connect(G_OBJECT(view), "advance_view",
+		     G_CALLBACK(advance_view_cb), paned_notebook);
+    g_signal_connect(G_OBJECT(view), "back_view",
+		     G_CALLBACK(back_view_cb), paned_notebook);
+    g_signal_connect(G_OBJECT(view), "toggle_mark",
+		     G_CALLBACK(toggle_mark_cb), application);
+    g_signal_connect(G_OBJECT(view), "write_response",
+		     G_CALLBACK(write_response_cb), application);
+    g_signal_connect(G_OBJECT(view), "close_threadlist",
+		     G_CALLBACK(close_threadlist_cb), application);
+    g_signal_connect(G_OBJECT(view), "toggle_hide",
+		     G_CALLBACK(toggle_hide_cb), application);
+    g_signal_connect(G_OBJECT(view), "reset_thread",
+		     G_CALLBACK(reset_thread_cb), application);
+    g_signal_connect(G_OBJECT(view), "copy_thread_url",
+		     G_CALLBACK(copy_thread_url_cb), application);
+    g_signal_connect(G_OBJECT(view), "mark_thread",
+		     G_CALLBACK(mark_thread_cb), application);
+    g_signal_connect(G_OBJECT(view), "thread_mouse_over",
+		     G_CALLBACK(thread_mouse_over_cb), application);
+    g_signal_connect(G_OBJECT(view), "thread_mouse_out",
+		     G_CALLBACK(thread_mouse_out_cb), application);
 
-      g_signal_connect(G_OBJECT(paned_notebook), "item_view_required",
-		       G_CALLBACK(item_view_required_cb), application);
-      g_signal_connect(G_OBJECT(paned_notebook), "item_view_being_closed",
-		       G_CALLBACK(item_view_being_closed_cb), application);
-      g_signal_connect(G_OBJECT(paned_notebook), "item_view_closed",
-		       G_CALLBACK(item_view_closed_cb), application);
-      g_signal_connect(G_OBJECT(paned_notebook), "page_switched",
-		       G_CALLBACK(page_switched_cb), application);
+    g_signal_connect(G_OBJECT(paned_notebook), "item_view_required",
+		     G_CALLBACK(item_view_required_cb), application);
+    g_signal_connect(G_OBJECT(paned_notebook), "item_view_being_closed",
+		     G_CALLBACK(item_view_being_closed_cb), application);
+    g_signal_connect(G_OBJECT(paned_notebook), "item_view_closed",
+		     G_CALLBACK(item_view_closed_cb), application);
+    g_signal_connect(G_OBJECT(paned_notebook), "page_switched",
+		     G_CALLBACK(page_switched_cb), application);
 
-      threadlist_view_set_enable_native_search(THREADLIST_VIEW(view), FALSE);
-      threadlist_view_set_title_match_func(THREADLIST_VIEW(view),
+    threadlist_view_set_enable_native_search(THREADLIST_VIEW(view), FALSE);
+    threadlist_view_set_title_match_func(THREADLIST_VIEW(view),
 			(ThreadlistViewTitleMatchFunc *)title_match_func);
 
-      g_signal_connect(G_OBJECT(view), "interactive_search",
-		       G_CALLBACK(interactive_search_cb), application);
+    g_signal_connect(G_OBJECT(view), "interactive_search",
+		     G_CALLBACK(interactive_search_cb), application);
 
-      job_args->application = application;
-      job_args->board = board;
-      job_args->view = THREADLIST_VIEW(view);
-      job_args->tab_label = tab_label;
-      job_args->increment_ignored = TRUE;
+    job_args->application = application;
+    job_args->board = board;
+    job_args->view = THREADLIST_VIEW(view);
+    job_args->tab_label = tab_label;
+    job_args->increment_ignored = TRUE;
 
-      job->canceled = FALSE;
-      job->job = show_threadlist;
-      job->args = job_args;
+    job->canceled = FALSE;
+    job->job = show_threadlist;
+    job->args = job_args;
 
-      info->is_busy = TRUE;
-      info->threadlist_source_buffer
-	= ochusha_bulletin_board_get_threadlist_source(board,
+    info->is_busy = TRUE;
+    info->threadlist_source_buffer
+      = ochusha_bulletin_board_get_threadlist_source(board,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
-      if (info->threadlist_source_buffer != NULL)
-	setup_for_tab_label_animation(info->threadlist_source_buffer,
-				      tab_label);
+    if (info->threadlist_source_buffer != NULL)
+      setup_for_tab_label_animation(info->threadlist_source_buffer,
+				    tab_label);
 
-      /* Ȥޤǳ */
-      g_object_ref(G_OBJECT(tab_label));
-      g_object_ref(G_OBJECT(view));
-      if (info->threadlist_source_buffer != NULL)
-	g_object_ref(G_OBJECT(info->threadlist_source_buffer));
+    /* Ȥޤǳ */
+    g_object_ref(G_OBJECT(tab_label));
+    g_object_ref(G_OBJECT(view));
+    if (info->threadlist_source_buffer != NULL)
+      g_object_ref(G_OBJECT(info->threadlist_source_buffer));
 #if DEBUG_WIDGET
-      fprintf(stderr, "threadlist_view->ref_count=%d\n",
-	      G_OBJECT(view)->ref_count);
+    fprintf(stderr, "threadlist_view->ref_count=%d\n",
+	    G_OBJECT(view)->ref_count);
 #endif
-      commit_job(job);
-    }
+    commit_job(job);
+  }
   THREAD_LIST_UNLOCK;
 
   return paned_notebook;
@@ -711,11 +743,10 @@ typedef struct _OpenThreadJobArgs
 {
   OchushaApplication *application;
   OchushaBulletinBoard *board;
-#if 0	/* XXX: not used */
-  OchushaBBSThread *thread;
-#endif
+
   char *thread_id;
   int res_num;
+
   gboolean in_tab;
 } OpenThreadJobArgs;
 
@@ -747,11 +778,10 @@ ochusha_open_thread(OchushaApplication *application,
 
   args->application = application;
   args->board = board;
-#if 0	/* XXX: not used */
-  args->thread = thread;
-#endif
+
   args->thread_id = G_STRDUP(thread_id);
   args->res_num = res_num;
+
   args->in_tab = in_tab;
 
   commit_job(job);
@@ -767,31 +797,44 @@ open_thread_view(WorkerThread *employee, gpointer job_args)
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
 
   THREAD_LIST_LOCK
-    {
-      PanedNotebook *board_view;
-      OchushaBBSThread *thread;
-      if (info->is_busy)
+  {
+    PanedNotebook *board_view;
+    OchushaBBSThread *thread
+      = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
+						       args->thread_id);
+    while (info->is_busy && thread == NULL)
+      {
 	THREAD_LIST_COND_WAIT;
+	thread
+	  = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
+							   args->thread_id);
+      }
 
 #if DEBUG_GUI_MOST
-      fprintf(stderr, "Thread ID (e.g. DAT file name): \"%s\"\n",
-	      args->thread_id);
+    fprintf(stderr, "Thread ID (e.g. DAT file name): \"%s\"\n",
+	    args->thread_id);
+#endif
+    /* λǡ٤Ϻ줿Ȥݾڤ */
+#if 0
+    thread = g_hash_table_lookup(board->thread_table, args->thread_id);
 #endif
-      /* λǡ٤Ϻ줿Ȥݾڤ */
-      thread = g_hash_table_lookup(board->thread_table, args->thread_id);
-      gdk_threads_enter();
-      board_view = (PanedNotebook *)paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
 
-      /* TODO:
-       * (board_view != NULL && thread == NULL)ʤ顢¿ʬΥDAT
-       * 긵˥å夷ƤʬȤäƼɽǤ褦ˤ
-       * ޤargs->res_numλȤƻͤ褦
-       */
+    if (thread != NULL)
+      {
+	gdk_threads_enter();
+	board_view = (PanedNotebook *)paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
 
-      if (board_view != NULL && thread != NULL)
-	paned_notebook_open_item_view(board_view, thread, args->in_tab);
-      gdk_threads_leave();
-    }
+	/* TODO:
+	 * (board_view != NULL && thread == NULL)ʤ顢¿ʬΥDAT
+	 * 긵˥å夷ƤʬȤäƼɽǤ褦ˤ
+	 * ޤargs->res_numλȤƻͤ褦
+	 */
+
+	if (board_view != NULL)
+	  paned_notebook_open_item_view(board_view, thread, args->in_tab);
+	gdk_threads_leave();
+      }
+  }
   THREAD_LIST_UNLOCK;
 
   G_FREE(args->thread_id);
@@ -827,54 +870,54 @@ refresh_threadlist_view(OchushaApplication *application,
     = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(scrolled_window)));
 
   THREAD_LIST_LOCK
-    {
-      OchushaAsyncBuffer *old_buffer;
-      if (info->is_busy)
-	{
-	  THREAD_LIST_UNLOCK;
+  {
+    OchushaAsyncBuffer *old_buffer;
+    if (info->is_busy)
+      {
+	THREAD_LIST_UNLOCK;
 #if DEBUG_GUI_MOST
-	  fprintf(stderr, "board is busy.\n");
+	fprintf(stderr, "board is busy.\n");
 #endif
-	  return;
-	}
+	return;
+      }
 
-      old_buffer = info->threadlist_source_buffer;
+    old_buffer = info->threadlist_source_buffer;
 
-      info->threadlist_source_buffer
-	= ochusha_bulletin_board_get_threadlist_source(board,
+    info->threadlist_source_buffer
+      = ochusha_bulletin_board_get_threadlist_source(board,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
-      if (info->threadlist_source_buffer == NULL)
-	{
-	  info->threadlist_source_buffer = old_buffer;
-	  THREAD_LIST_UNLOCK;
-	  return;
-	}
+    if (info->threadlist_source_buffer == NULL)
+      {
+	info->threadlist_source_buffer = old_buffer;
+	THREAD_LIST_UNLOCK;
+	return;
+      }
 
-      setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
+    setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
 
-      job = G_NEW0(WorkerJob, 1);
-      job_args = G_NEW0(ThreadlistJobArgs, 1);
+    job = G_NEW0(WorkerJob, 1);
+    job_args = G_NEW0(ThreadlistJobArgs, 1);
 
-      job_args->application = application;
-      job_args->board = board;
-      job_args->view = threadlist_view;
-      job_args->tab_label = tab_label;
-      job_args->increment_ignored = TRUE;
+    job_args->application = application;
+    job_args->board = board;
+    job_args->view = threadlist_view;
+    job_args->tab_label = tab_label;
+    job_args->increment_ignored = TRUE;
       
-      job->canceled = FALSE;
-      job->job = show_threadlist;
-      job->args = job_args;
+    job->canceled = FALSE;
+    job->job = show_threadlist;
+    job->args = job_args;
       
-      info->is_busy = TRUE;
-      /* λޤǳ */
-      g_object_ref(G_OBJECT(info->threadlist_source_buffer));
-      g_object_ref(G_OBJECT(tab_label));
-      g_object_ref(G_OBJECT(threadlist_view));
-      commit_job(job);
+    info->is_busy = TRUE;
+    /* λޤǳ */
+    g_object_ref(G_OBJECT(info->threadlist_source_buffer));
+    g_object_ref(G_OBJECT(tab_label));
+    g_object_ref(G_OBJECT(threadlist_view));
+    commit_job(job);
 
-      g_object_unref(G_OBJECT(old_buffer));
-    }
+    g_object_unref(G_OBJECT(old_buffer));
+  }
   THREAD_LIST_UNLOCK;
 }
 
@@ -909,37 +952,40 @@ redraw_threadlist_view(OchushaApplication *application,
     = THREADLIST_VIEW(gtk_bin_get_child(GTK_BIN(scrolled_window)));
 
   THREAD_LIST_LOCK
-    {
+  {
+    if (info->is_busy)
+      {
+	info->redraw_required = TRUE;
+	THREAD_LIST_UNLOCK;
+	return;
+      }
 
-      while (info->is_busy)
-	THREAD_LIST_COND_WAIT;
+    if (info->threadlist_source_buffer == NULL)
+      {
+	THREAD_LIST_UNLOCK;
+	return;
+      }
 
-      if (info->threadlist_source_buffer == NULL)
-	{
-	  THREAD_LIST_UNLOCK;
-	  return;
-	}
+    job = G_NEW0(WorkerJob, 1);
+    job_args = G_NEW0(ThreadlistJobArgs, 1);
 
-      job = G_NEW0(WorkerJob, 1);
-      job_args = G_NEW0(ThreadlistJobArgs, 1);
-
-      job_args->application = application;
-      job_args->board = board;
-      job_args->view = threadlist_view;
-      job_args->tab_label = tab_label;
-      job_args->increment_ignored = FALSE;
+    job_args->application = application;
+    job_args->board = board;
+    job_args->view = threadlist_view;
+    job_args->tab_label = tab_label;
+    job_args->increment_ignored = FALSE;
       
-      job->canceled = FALSE;
-      job->job = show_threadlist;
-      job->args = job_args;
-      
-      info->is_busy = TRUE;
-      /* λޤǳ */
-      g_object_ref(G_OBJECT(info->threadlist_source_buffer));
-      g_object_ref(G_OBJECT(tab_label));
-      g_object_ref(G_OBJECT(threadlist_view));
-      commit_job(job);
-    }
+    job->canceled = FALSE;
+    job->job = show_threadlist;
+    job->args = job_args;
+
+    info->is_busy = TRUE;
+    /* λޤǳ */
+    g_object_ref(G_OBJECT(info->threadlist_source_buffer));
+    g_object_ref(G_OBJECT(tab_label));
+    g_object_ref(G_OBJECT(threadlist_view));
+    commit_job(job);
+  }
   THREAD_LIST_UNLOCK;
 }
 
@@ -1055,9 +1101,11 @@ show_threadlist(WorkerThread *employee, gpointer args)
     ochusha_bulletin_board_read_threadlist_xml(board, &application->config);
 
   THREAD_LIST_LOCK
-    {
-      buffer = info->threadlist_source_buffer;
-    }
+  {
+  redraw:
+    buffer = info->threadlist_source_buffer;
+    info->redraw_required = FALSE;
+  }
   THREAD_LIST_UNLOCK;
 
   if (buffer == NULL)
@@ -1111,10 +1159,15 @@ show_threadlist(WorkerThread *employee, gpointer args)
 	  G_OBJECT(buffer)->ref_count);
 #endif
   THREAD_LIST_LOCK
-    {
-      info->is_busy = FALSE;
-      THREAD_LIST_COND_BROADCAST;
-    }
+  {
+    if (!info->redraw_required)
+      {
+	info->is_busy = FALSE;
+	THREAD_LIST_COND_BROADCAST;
+      }
+    else
+      goto redraw;
+  }
   THREAD_LIST_UNLOCK;
 
   gdk_threads_enter();
@@ -1138,13 +1191,14 @@ snatch_threadlist_source_buffer_for_board(OchushaBulletinBoard *board,
   OchushaAsyncBuffer *buffer;
 
   THREAD_LIST_LOCK
-    {
-      buffer = info->threadlist_source_buffer;
-      info->is_busy = FALSE;
-      info->threadlist_source_buffer = NULL;
-      THREAD_LIST_COND_BROADCAST;
-    }
+  {
+    buffer = info->threadlist_source_buffer;
+    info->is_busy = FALSE;
+    info->threadlist_source_buffer = NULL;
+    THREAD_LIST_COND_BROADCAST;
+  }
   THREAD_LIST_UNLOCK;
+
   return buffer;
 }
 
@@ -1160,8 +1214,8 @@ update_threadlist_entry_style(OchushaApplication *application,
     {
       0,	/* flags */
       0,	/* reserved */
-      0,	/* foreground color */
-      0		/* background color */
+      LIST_ENTRY_FOREGROUND_NORMAL,	/* foreground color */
+      LIST_ENTRY_BACKGROUND_NORMAL	/* background color */
     };
 
   gboolean exist_new_responses
@@ -1174,24 +1228,24 @@ update_threadlist_entry_style(OchushaApplication *application,
   if (info->view_flags & BBS_THREAD_FAVORITE)
     visual_parameter.flags |= THREADLIST_ENTRY_FLAG_MARK;
 
-  /* foreground */
   if (info->view_flags & BBS_THREAD_HIDDEN)
-    visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_COLOR_GRAY75;
-  else if (exist_new_responses)
     {
-      if (info->view_flags & BBS_THREAD_FAVORITE)
-	visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_COLOR_RED;
+      visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_HIDDEN;
+      visual_parameter.background_color = LIST_ENTRY_BACKGROUND_HIDDEN;
+    }
+  else if (info->view_flags & BBS_THREAD_NEW)
+    {
+      visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_STRONG;
+      visual_parameter.background_color = LIST_ENTRY_BACKGROUND_STRONG;
+    }
+  else if (threadlist_filter_check_thread(&board_info->filter,
+					  application, thread))
+    {
+      if (exist_new_responses)
+	visual_parameter.foreground_color = LIST_ENTRY_FOREGROUND_EMPH;
+      if (!board_info->enable_filter)
+	visual_parameter.background_color = LIST_ENTRY_BACKGROUND_EMPH;
     }
-
-  /* background */
-  if (info->view_flags & BBS_THREAD_NEW)
-    visual_parameter.background_color = LIST_ENTRY_BACKGROUND_COLOR_PINK;
-  else if (exist_new_responses
-	   && threadlist_filter_check_thread(&board_info->filter,
-					     application,
-					     thread))
-    visual_parameter.background_color
-      = LIST_ENTRY_BACKGROUND_COLOR_LAVENDER;
 
   threadlist_view_update_thread(view, thread);
   threadlist_view_update_thread_visual(view, thread, &visual_parameter);
@@ -1236,6 +1290,24 @@ back_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
 }
 
 
+static void
+write_response_cb(ThreadlistView *view, OchushaBBSThread *thread,
+		  OchushaApplication *application)
+{
+  /* ˤ륫֤Υ˥쥹񤯤Ȥ񤤤
+   * 
+   */
+  write_response(application);
+}
+
+
+static void
+close_threadlist_cb(ThreadlistView *view, OchushaApplication *application)
+{
+  paned_notebook_close_current_item_view(PANED_NOTEBOOK(application->contents_window));
+}
+
+
 static void
 toggle_mark_cb(ThreadlistView *view, OchushaBBSThread *thread,
 	       OchushaApplication *application)
@@ -1305,14 +1377,14 @@ thread_mouse_over_cb(ThreadlistView *view, OchushaBBSThread *thread,
 #endif
 
   THREAD_LIST_LOCK
-    {
-      if (title_popup_delay_id != 0)
-	g_source_remove(title_popup_delay_id);
-      popup_thread = thread;
-      title_popup_delay_id = g_timeout_add(application->popup_title_delay,
-					   thread_title_popup_timeout,
-					   thread);
-    }
+  {
+    if (title_popup_delay_id != 0)
+      g_source_remove(title_popup_delay_id);
+    popup_thread = thread;
+    title_popup_delay_id = g_timeout_add(application->popup_title_delay,
+					 thread_title_popup_timeout,
+					 thread);
+  }
   THREAD_LIST_UNLOCK;
 }
 
@@ -1325,11 +1397,11 @@ thread_mouse_out_cb(ThreadlistView *view, OchushaBBSThread *thread,
   fprintf(stderr, "thread_mouse_out_cb: thread=%p\n", thread);
 #endif
   THREAD_LIST_LOCK
-    {
-      if (title_popup_delay_id != 0)
-	g_source_remove(title_popup_delay_id);
-      popup_thread = NULL;
-    }
+  {
+    if (title_popup_delay_id != 0)
+      g_source_remove(title_popup_delay_id);
+    popup_thread = NULL;
+  }
   THREAD_LIST_UNLOCK;
 
   gtk_widget_hide_all(title_popup_window);
@@ -1350,14 +1422,14 @@ thread_title_popup_timeout(gpointer data)
   if (data == popup_thread)
     {
       THREAD_LIST_LOCK
-        {
-	  thread_title_popup_show();
+      {
+	thread_title_popup_show();
 
-	  if (title_popup_delay_id != 0)
-	    title_popup_delay_id = 0;
+	if (title_popup_delay_id != 0)
+	  title_popup_delay_id = 0;
 
-	  popup_thread = NULL;
-	}
+	popup_thread = NULL;
+      }
       THREAD_LIST_UNLOCK;
 
       result = FALSE;
@@ -1412,6 +1484,9 @@ item_view_required_cb(PanedNotebook *paned_notebook,
   thread_view = open_bbs_thread(application, thread, ICON_LABEL(*tab_label));
   if (thread_view != NULL)
     {
+      g_signal_connect_swapped(G_OBJECT(thread_view), "close_thread",
+			       G_CALLBACK(paned_notebook_close_current_item_view),
+			       paned_notebook);
       gtk_widget_show(thread_view);
       *item_view = gtk_scrolled_window_new(NULL, NULL);
       gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(*item_view),
@@ -1472,7 +1547,6 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
 	  thread, item_view);
 #endif
 
-#ifndef SUSPEND_ALL_FOR_DESTRUCTION	/* ¸ */
   /*
    * MEMO: ɽƤ륦åȤºݤ˼ΤƤ륿ߥ󥰤򤺤餹
    * 
@@ -1501,30 +1575,6 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
 
       gdk_threads_enter();
     }
-#else
-  /*
-   * ΥɽĤ褦ȤƤΤǡϢ륹åɤߤᡢ
-   * ¾GUIϢΥåɤ򥵥ڥɤίäƤ륤٥Ȥ
-   * Ƥ롣ǽʸ¤᤯٥Ȥò뤿ᡢƤthread
-   * ڥɤˡ˴Ϣthread϶Ūߤ롣
-   *
-   * ġĤ褦ˤ顢ब򤱤褦ʵΤġġ
-   */
-
-  gdk_threads_leave();
-
-  ochusha_async_buffer_suspend_all();
-
-  if (info->response_source_buffer != NULL)
-    ochusha_async_buffer_terminate(info->response_source_buffer);
-
-  gdk_threads_enter();
-
-  while (gtk_events_pending())
-    gtk_main_iteration();
-
-  ochusha_async_buffer_resume_all();
-#endif
 
   if (item_view != NULL)
     {
@@ -1545,9 +1595,7 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *thread,
   fprintf(stderr, "closed: thread=%p, item_view=%p\n", thread, item_view);
 #endif
 
-#ifndef SUSPEND_ALL_FOR_DESTRUCTION
   delayed_g_object_unref(G_OBJECT(item_view));
-#endif
 
   info->last_read = NULL;
   info->next_mark = NULL;
@@ -1695,126 +1743,174 @@ reverse_strpbrk(const gchar *string, const gchar *charset)
 }
 
 
+typedef struct _WriteResponseJobArgs
+{
+  OchushaBBSThread *thread;
+  gchar *name;
+  gchar *mailto;
+  gchar *message;
+  GtkWidget *dialog;
+  OchushaApplication *application;
+} WriteResponseJobArgs;
+
+
+static void
+background_write_response(WorkerThread *employee, gpointer job_args)
+{
+  WriteResponseJobArgs *args = (WriteResponseJobArgs *)job_args;
+  OchushaBBSResponse response;
+  gboolean result;
+
+  response.name = args->name;
+  response.mailto = args->mailto;
+  response.content = args->message;
+
+  result = ochusha_bbs_thread_post_response(args->thread,
+					    args->application->broker,
+					    &response);
+
+  if (result)
+    {
+      BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(args->thread);
+      BulletinBoardGUIInfo *board_info
+	= ensure_bulletin_board_info(args->thread->board, args->application);
+      gboolean number_only
+	= (response.name[0] != '\0'
+	   && reverse_strpbrk(response.name, "0123456789") == NULL);
+      GtkWidget *notebook;
+
+      gdk_threads_enter();
+      if (thread_info->last_name == NULL
+	  || strcmp(thread_info->last_name, response.name) != 0)
+	{
+	  if (thread_info->last_name != NULL)
+	    G_FREE(thread_info->last_name);
+	  thread_info->last_name = G_STRDUP(response.name);
+	}
+
+      if (thread_info->last_mail == NULL
+	  || strcmp(thread_info->last_mail, response.mailto) != 0)
+	{
+	  if (thread_info->last_mail != NULL)
+	    G_FREE(thread_info->last_mail);
+	  thread_info->last_mail = G_STRDUP(response.mailto);
+	}
+
+      if (!number_only)
+	{
+	  if (board_info->last_name == NULL
+	      || strcmp(board_info->last_name, response.name) != 0)
+	    {
+	      if (board_info->last_name != NULL)
+		G_FREE(board_info->last_name);
+	      board_info->last_name = G_STRDUP(response.name);
+	    }
+
+	  if (board_info->last_mail == NULL
+	      || strcmp(board_info->last_mail, response.mailto) != 0)
+	    {
+	      if (board_info->last_mail != NULL)
+		G_FREE(board_info->last_mail);
+	      board_info->last_mail = G_STRDUP(response.mailto);
+	    }
+
+	  if (args->application->last_name == NULL
+	      || strcmp(args->application->last_name, response.name) != 0)
+	    {
+	      if (args->application->last_name != NULL)
+		G_FREE(args->application->last_name);
+	      args->application->last_name = G_STRDUP(response.name);
+	    }
+
+	  if (args->application->last_mail == NULL
+	      || strcmp(args->application->last_mail, response.mailto) != 0)
+	    {
+	      if (args->application->last_mail != NULL)
+		G_FREE(args->application->last_mail);
+	      args->application->last_mail = G_STRDUP(response.mailto);
+	    }
+	}
+
+      notebook = paned_notebook_get_item_view(PANED_NOTEBOOK(args->application->contents_window), args->thread->board);
+      if (notebook != NULL)
+	{
+	  GtkWidget *view = paned_notebook_get_item_view(PANED_NOTEBOOK(notebook), args->thread);
+	  GtkWidget *label = paned_notebook_get_tab_label(PANED_NOTEBOOK(notebook), args->thread);
+	  if (view != NULL && label != NULL)
+	    {
+	      GtkWidget *thread_view = gtk_bin_get_child(GTK_BIN(view));
+	      refresh_thread(args->application, thread_view, args->thread,
+			     ICON_LABEL(label));
+	    }
+	}
+
+      gtk_widget_destroy(args->dialog);
+      gdk_threads_leave();
+    }
+  else
+    {
+      gdk_threads_enter();
+      ochusha_open_url(args->application,
+		       ochusha_bbs_thread_get_url_to_post_response(args->thread),
+		       FALSE, TRUE);
+      gtk_widget_show_all(args->dialog);
+      gdk_threads_leave();
+    }
+
+  G_FREE(args->name);
+  G_FREE(args->mailto);
+  G_FREE(args->message);
+  G_FREE(args);
+}
+
+
 static void
 write_dialog_response_cb(GtkWidget *write_dialog, gint response_id,
 			 OchushaApplication *application)
 {
   OchushaBBSThread *thread = g_object_get_data(G_OBJECT(write_dialog),
 					       "thread");
-  gboolean result;
+  ResponseEditor *editor;
+  const gchar *name;
+  const gchar *mailto;
+  gchar *message;
 
-  if (response_id == GTK_RESPONSE_OK)
-    {
-      ResponseEditor *editor = g_object_get_data(G_OBJECT(write_dialog),
-						 "editor");
-      gchar *message;
-      OchushaBBSResponse response;
-
-      response.name = response_editor_get_name(editor);
-      response.mailto = response_editor_get_mail(editor);
-      message = response_editor_get_response(editor);
-      response.content = message;
-
-      if (message != NULL && *message != '\0')
-	{
-	  result = ochusha_bbs_thread_post_response(thread,
-						    application->broker,
-						    &response);
-	  g_free(message);
-	  if (result)
-	    {
-	      BBSThreadGUIInfo *thread_info = ensure_bbs_thread_info(thread);
-	      BulletinBoardGUIInfo *board_info
-		= ensure_bulletin_board_info(thread->board, application);
-	      gboolean number_only
-		= (response.name[0] != '\0'
-		   && reverse_strpbrk(response.name, "0123456789") == NULL);
-
-	      if (thread_info->last_name == NULL
-		  || strcmp(thread_info->last_name, response.name) != 0)
-		{
-		  if (thread_info->last_name != NULL)
-		    G_FREE(thread_info->last_name);
-		  thread_info->last_name = G_STRDUP(response.name);
-		}
-
-	      if (thread_info->last_mail == NULL
-		  || strcmp(thread_info->last_mail, response.mailto) != 0)
-		{
-		  if (thread_info->last_mail != NULL)
-		    G_FREE(thread_info->last_mail);
-		  thread_info->last_mail = G_STRDUP(response.mailto);
-		}
-
-	      if (!number_only)
-		{
-		  if (board_info->last_name == NULL
-		      || strcmp(board_info->last_name, response.name) != 0)
-		    {
-		      if (board_info->last_name != NULL)
-			G_FREE(board_info->last_name);
-		      board_info->last_name = G_STRDUP(response.name);
-		    }
-
-		  if (board_info->last_mail == NULL
-		      || strcmp(board_info->last_mail, response.mailto) != 0)
-		    {
-		      if (board_info->last_mail != NULL)
-			G_FREE(board_info->last_mail);
-		      board_info->last_mail = G_STRDUP(response.mailto);
-		    }
-
-		  if (application->last_name == NULL
-		      || strcmp(application->last_name, response.name) != 0)
-		    {
-		      if (application->last_name != NULL)
-			G_FREE(application->last_name);
-		      application->last_name = G_STRDUP(response.name);
-		    }
-
-		  if (application->last_mail == NULL
-		      || strcmp(application->last_mail, response.mailto) != 0)
-		    {
-		      if (application->last_mail != NULL)
-			G_FREE(application->last_mail);
-		      application->last_mail = G_STRDUP(response.mailto);
-		    }
-		}
-	    }
-	}
-      else
-	{
-	  /* ʸʤϥ顼ˤʤ餷 */
-	  result = TRUE;
-	  response_id = GTK_RESPONSE_CANCEL;
-	}
-    }
-  else
-    result = TRUE;
-
-  if (result)
+  if (response_id != GTK_RESPONSE_OK)
     {
       gtk_widget_destroy(write_dialog);
-      if (response_id == GTK_RESPONSE_OK)
-	{
-	  GtkWidget *notebook = paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), thread->board);
-	  if (notebook != NULL)
-	    {
-	      GtkWidget *view = paned_notebook_get_item_view(PANED_NOTEBOOK(notebook), thread);
-	      GtkWidget *label = paned_notebook_get_tab_label(PANED_NOTEBOOK(notebook), thread);
-	      if (view != NULL && label != NULL)
-		{
-		  GtkWidget *thread_view = gtk_bin_get_child(GTK_BIN(view));
-		  refresh_thread(application, thread_view, thread,
-				 ICON_LABEL(label));
-		}
-	    }
-	}
+      return;
     }
-  else
-    ochusha_open_url(application,
-		     ochusha_bbs_thread_get_url_to_post_response(thread),
-		     FALSE, TRUE);
+
+  editor = g_object_get_data(G_OBJECT(write_dialog), "editor");
+
+  name = response_editor_get_name(editor);
+  mailto = response_editor_get_mail(editor);
+  message = response_editor_get_response(editor);
+
+  if (message != NULL && *message != '\0')
+    {
+      WorkerJob *job = G_NEW0(WorkerJob, 1);
+      WriteResponseJobArgs *job_args = G_NEW0(WriteResponseJobArgs, 1);
+      
+      job_args->thread = thread;
+      job_args->name = G_STRDUP(name);
+      job_args->mailto = G_STRDUP(mailto);
+      job_args->message = message;
+      job_args->dialog = write_dialog;
+      job_args->application = application;
+
+      job->canceled = FALSE;
+      job->job = background_write_response;
+      job->args = job_args;
+
+      gtk_widget_hide(write_dialog);
+
+      commit_job(job);
+
+      return;
+    }
+
+  G_FREE(message);
 }
 
 
@@ -2006,9 +2102,8 @@ setup_thread_search_window(OchushaApplication *application)
 }
 
 
-static void
-start_thread_search_button_cb(GtkWidget *widget,
-			      OchushaApplication *application)
+void
+start_thread_search(OchushaApplication *application)
 {
   if (application->thread_search_window == NULL)
     setup_thread_search_window(application);
@@ -2018,6 +2113,14 @@ start_thread_search_button_cb(GtkWidget *widget,
 }
 
 
+static void
+start_thread_search_button_cb(GtkWidget *widget,
+			      OchushaApplication *application)
+{
+  start_thread_search(application);
+}
+
+
 void
 jump_to_new_comer_response_of_current_thread(OchushaApplication *application)
 {
