@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.14 2003/11/23 21:15:48 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.17 2003/12/04 07:32:17 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -90,6 +90,7 @@ typedef struct _BulletinBoardGUIInfo
 
   gboolean enable_filter;
   volatile gboolean is_busy;
+  volatile gboolean redraw_required;
 } BulletinBoardGUIInfo;
 
 
@@ -120,11 +121,12 @@ GtkWidget *open_bulletin_board(OchushaApplication *application,
 			       IconLabel *tab_label);
 void finalize_bulletin_board_view(OchushaBulletinBoard *board,
 				  GtkWidget *widget);
-
 void ochusha_open_thread(OchushaApplication *application,
 			 OchushaBulletinBoard *board, const char *thread_id,
 			 int res_num, gboolean in_tab);
 
+void write_response(OchushaApplication *application);
+
 void refresh_threadlist_view(OchushaApplication *application,
 			     OchushaBulletinBoard *board);
 void redraw_threadlist_view(OchushaApplication *application,
@@ -137,6 +139,7 @@ void update_threadlist_entry_style(OchushaApplication *application,
 				   OchushaBBSThread *thread);
 
 void refresh_current_thread(OchushaApplication *application);
+void start_thread_search(OchushaApplication *application);
 void jump_to_new_comer_response_of_current_thread(OchushaApplication *application);
 void go_to_the_last_response_of_current_thread(OchushaApplication *application);
 void go_to_the_first_response_of_current_thread(OchushaApplication *application);
