@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: icon_label.h,v 1.1.1.1 2003/05/10 16:34:25 fuyu Exp $
+ * $Id: icon_label.h,v 1.3 2003/05/14 18:47:40 fuyu Exp $
  */
 
 #ifndef _ICON_LABEL_H_
@@ -50,34 +50,24 @@ typedef struct _IconLabelClass IconLabelClass;
 
 struct _IconLabel
 {
-  GtkHBox container;
+  GtkLabel label;
+  GtkAllocation fake_allocation;
 
-  GtkLabel *label;
-
-  int current_icon;
-
-  int number_of_icons;
-  GtkWidget **icon_list;
-
-  int counter;	/* ȿ§ݤʤΤǤ֤㤪 */
+  GdkPixbuf *icon;
+  gint icon_y_adjustment;
 };
 
 
 struct _IconLabelClass
 {
-  GtkHBoxClass parent_class;
-
-  void (*alternate_icon)(IconLabel *label, gint icon_number);
+  GtkLabelClass parent_class;
 };
 
 
 GType icon_label_get_type(void);
 
-GtkWidget *icon_label_new(const gchar *label_text, GtkWidget *icon_list[],
-			 int number_of_icons, int initial_icon);
-
-void icon_label_set_label_text(IconLabel *icon_label, const gchar *label_text);
-void icon_label_alternate_icon(IconLabel *icon_label, int icon_number);
+GtkWidget *icon_label_new(const gchar *label_text, GdkPixbuf *icon);
+void icon_label_set_icon(IconLabel *icon_label, GdkPixbuf *icon);
 
 
 #ifdef __cplusplus
