@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.32 2003/11/26 06:03:08 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.42 2003/12/03 12:01:10 fuyu Exp $
  */
 
 #include "config.h"
@@ -57,6 +57,7 @@
 
 #include <time.h>
 
+#include "write_response_24.h"
 
 #include "blue0.xpm"
 #include "blue1.xpm"
@@ -66,17 +67,13 @@
 #include "black.xpm"
 #include "red.xpm"
 
-#include "ochusha48.xpm"
+#include "ochusha48.h"
 
-#if USE_BANNER_XPM
-#include "ochusha-banner.xpm"
-#include "ochusha-banner1.xpm"
-#else
 #include "ochusha-banner.h"
 #include "ochusha-banner1.h"
-#endif
 
 
+static void setup_ochusha_stock_items(void);
 static void setup_tab_label_icons(void);
 static void delete_event_cb(GtkWidget *widget, GdkEventAny *event,
 			    gpointer user_data);
@@ -118,6 +115,9 @@ static gboolean access_failed_cb(OchushaNetworkBroker *broker,
 static void output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
 			  OchushaApplication *application);
 
+#if ENABLE_DEBUG_CONSOLE
+static void open_debug_console(OchushaApplication *application);
+#endif
 static void open_preference_dialog(OchushaApplication *application);
 static void open_about_dialog(OchushaApplication *application);
 static void about_dialog_response_cb(GtkWidget *dialog, gint response_id,
@@ -161,6 +161,12 @@ static GtkWidget *open_url_combo = NULL;
 static GtkWidget *open_url_in_tab_button = NULL;
 static GList *open_url_history = NULL;
 
+#if ENABLE_DEBUG_CONSOLE
+static GtkWidget *debug_console = NULL;
+static GtkTextBuffer *log_buffer = NULL;
+static GtkTextIter log_tail_iter;
+#endif
+
 
 void
 ochusha_init_application(OchushaApplication *application)
@@ -224,7 +230,7 @@ ochusha_init_application(OchushaApplication *application)
     },
     {
       _("/Thread/_Refresh"),
-      NULL,
+      "F5",
       main_menu_callback,
       MAIN_MENU_REFRESH_THREAD,
       NULL
@@ -244,7 +250,7 @@ ochusha_init_application(OchushaApplication *application)
       "<Separator>"
     },
     {
-      _("/Thread/Refresh Threadlist"),
+      _("/Thread/Refresh _Threadlist"),
       NULL,
       main_menu_callback,
       MAIN_MENU_REFRESH_THREADLIST,
@@ -271,7 +277,7 @@ ochusha_init_application(OchushaApplication *application)
       0,
       "<Branch>"
     },
-#if 0
+#if ENABLE_DEBUG_CONSOLE
     {
       _("/Tools/Debug Console"),
       NULL,
@@ -321,19 +327,13 @@ ochusha_init_application(OchushaApplication *application)
       abort();
     }
 
+  setup_ochusha_stock_items();
   setup_tab_label_icons();
 
-#if USE_BANNER_XPM
-  banner_pixbuf
-    = gdk_pixbuf_new_from_xpm_data((const char **)ochusha_banner_xpm);
-  banner1_pixbuf
-    = gdk_pixbuf_new_from_xpm_data((const char **)ochusha_banner1_xpm);
-#else
   banner_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner_inline, FALSE, NULL);
   banner1_pixbuf
     = gdk_pixbuf_new_from_inline(-1, ochusha_banner1_inline, FALSE, NULL);
-#endif
 
   configure_application(application);
 
@@ -348,8 +348,7 @@ ochusha_init_application(OchushaApplication *application)
   gtk_window_set_title(application->top_level, _("ochusha"));
 
   gtk_window_set_icon(application->top_level,
-		      gdk_pixbuf_new_from_xpm_data((const char **)ochusha48_xpm));
-
+		      gdk_pixbuf_new_from_inline(-1, ochusha48_inline, FALSE, NULL));
 
   application->main_window = GTK_BOX(gtk_vbox_new(FALSE, 0));
   gtk_container_add(GTK_CONTAINER(application->top_level),
@@ -492,6 +491,11 @@ ochusha_init_application(OchushaApplication *application)
 
   application->clipboard = NULL;
 
+#if ENABLE_DEBUG_CONSOLE
+  log_buffer = gtk_text_buffer_new(NULL);
+  gtk_text_buffer_get_start_iter(log_buffer, &log_tail_iter);
+#endif
+
   gtk_widget_show_all(GTK_WIDGET(application->top_level));
 }
 
@@ -512,14 +516,14 @@ void
 ochusha_clipboard_set_text(OchushaApplication *application, const gchar *text)
 {
   MAIN_UI_LOCK
-    {
-      if (application->clipboard == NULL)
-	application->clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+  {
+    if (application->clipboard == NULL)
+      application->clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
 #if DEBUG_GUI_MOST
-      fprintf(stderr, "setting text on clipboard: \"%s\"\n", text);
+    fprintf(stderr, "setting text on clipboard: \"%s\"\n", text);
 #endif
-      gtk_clipboard_set_text(application->clipboard, text, -1);
-    }
+    gtk_clipboard_set_text(application->clipboard, text, -1);
+  }
   MAIN_UI_UNLOCK
 }
 
@@ -535,12 +539,9 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
   char *thread_id = NULL;
   unsigned int from = 0;
   unsigned int to = 0;
+  char *scheme;
   char command_line[COMMAND_LINE_BUFFER_SIZE];
 
-  /*
-   * XXX: Ǥurl2chURLɤĴ٤Ƥۤˤ餷Ƥ뤬
-   *      ʥ2ch¸ʬϤ٤
-   */
   if (!use_web_browser
       && ochusha_bbs_table_check_url(application->table, url,
 				     &board, &board_url, &bbs_type,
@@ -586,11 +587,23 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
   if (thread_id != NULL)
     G_FREE(thread_id);
 
+  scheme = ochusha_utils_url_extract_scheme(url);
+
+  if (scheme == NULL)
+    return;
+
+  /* TODO: scheme̤˵ư볰ץѹǽˤ٤ */
+
+  G_FREE(scheme);
+
   if (snprintf(command_line, COMMAND_LINE_BUFFER_SIZE,
 	       application->web_browser_template, url)
       < COMMAND_LINE_BUFFER_SIZE)
     {
       GError *error = NULL;
+      char *native_command_line
+	= g_locale_from_utf8(command_line, -1, NULL, NULL, NULL);
+
       /*
        * MEMO: UTF-8Ǥ'~'E280BEˤʤäƤꡢ줬ޤޤ줿URL
        *       mozillaϤƤŪΥڡɽʤȤ
@@ -602,8 +615,6 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
        * XXX:  UTF-8ʸnativeencodingѴǽȤϸ¤ʤ
        *       Τϥ顼åȤкפ롣
        */
-      char *native_command_line
-	= g_locale_from_utf8(command_line, -1, NULL, NULL, NULL);
       if (!g_spawn_command_line_async(native_command_line, &error))
 	{
 	  /* TODO: 顼ɽ */
@@ -653,6 +664,9 @@ setup_for_tab_label_animation(OchushaAsyncBuffer *buffer, IconLabel *tab_label)
 		   G_CALLBACK(invalidate_tab_label_animation),
 		   buffer);
   g_object_set_qdata(G_OBJECT(buffer), tab_label_id, tab_label);
+
+  if (buffer->fixed)
+    icon_label_set_icon(tab_label, tab_label_icons[TAB_LABEL_ICON_DONE]);
 }
 
 
@@ -679,23 +693,120 @@ get_tab_label(const gchar *label_text)
 }
 
 
+static void
+add_stock_icon(GtkIconFactory *factory, const char *stock_id,
+	       GdkPixbuf *pixbuf, GtkIconSize size)
+{
+  GtkIconSource *source = gtk_icon_source_new();
+  GtkIconSet *set = gtk_icon_set_new();
+
+  gtk_icon_source_set_size(source, size);
+  gtk_icon_source_set_pixbuf(source, pixbuf);
+
+#if 0	/* ׽ */
+  gtk_icon_source_set_size_wildcarded(source, FALSE);
+  gtk_icon_set_add_source(set, source);
+#endif
+
+  gtk_icon_source_set_size_wildcarded(source, TRUE);
+  gtk_icon_set_add_source(set, source);
+
+  gtk_icon_source_free(source);
+
+  gtk_icon_factory_add(factory, stock_id, set);
+
+  gtk_icon_set_unref(set);
+  g_object_unref(pixbuf);	/* ݤʤΤǤunrefƤޤ */
+}
+
+
+static const GtkStockItem ochusha_stock_items[] =
+{
+  { OCHUSHA_STOCK_WRITE_RESPONSE, N_("_Write"), 0, 0, PACKAGE_NAME }
+};
+
+
+static void
+setup_ochusha_stock_items(void)
+{
+  GtkIconFactory *factory = gtk_icon_factory_new();
+
+  add_stock_icon(factory, OCHUSHA_STOCK_WRITE_RESPONSE,
+		 gdk_pixbuf_new_from_inline(-1, write_response_24,
+					    FALSE, NULL),
+		 GTK_ICON_SIZE_LARGE_TOOLBAR);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS0,
+		 gdk_pixbuf_new_from_xpm_data((const char **)blue0_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS1,
+		 gdk_pixbuf_new_from_xpm_data((const char **)blue1_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS2,
+		 gdk_pixbuf_new_from_xpm_data((const char **)blue2_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_PROGRESS3,
+		 gdk_pixbuf_new_from_xpm_data((const char **)blue3_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_PREPARATION,
+		 gdk_pixbuf_new_from_xpm_data((const char **)orange_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_DONE,
+		 gdk_pixbuf_new_from_xpm_data((const char **)black_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  add_stock_icon(factory, OCHUSHA_STOCK_NET_ERROR,
+		 gdk_pixbuf_new_from_xpm_data((const char **)red_xpm),
+		 GTK_ICON_SIZE_MENU);
+
+  gtk_icon_factory_add_default(factory);
+
+  g_object_unref(factory);
+
+  gtk_stock_add_static(ochusha_stock_items, G_N_ELEMENTS(ochusha_stock_items));
+}
+
+
 static void
 setup_tab_label_icons(void)
 {
+  GtkWidget *dummy = icon_label_new("", NULL);
+
   tab_label_icons[TAB_LABEL_ICON_PROGRESS0]
-    = gdk_pixbuf_new_from_xpm_data((const char **)blue0_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_PROGRESS0,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-progress-0");
+
   tab_label_icons[TAB_LABEL_ICON_PROGRESS1]
-    = gdk_pixbuf_new_from_xpm_data((const char **)blue1_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_PROGRESS1,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-progress-1");
+
   tab_label_icons[TAB_LABEL_ICON_PROGRESS2]
-    = gdk_pixbuf_new_from_xpm_data((const char **)blue2_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_PROGRESS2,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-progress-2");
+
   tab_label_icons[TAB_LABEL_ICON_PROGRESS3]
-    = gdk_pixbuf_new_from_xpm_data((const char **)blue3_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_PROGRESS3,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-progress-3");
+
   tab_label_icons[TAB_LABEL_ICON_PREPARATION]
-    = gdk_pixbuf_new_from_xpm_data((const char **)orange_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_PREPARATION,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-preparation");
+
   tab_label_icons[TAB_LABEL_ICON_DONE]
-    = gdk_pixbuf_new_from_xpm_data((const char **)black_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_DONE,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-done");
+
   tab_label_icons[TAB_LABEL_ICON_ERROR]
-    = gdk_pixbuf_new_from_xpm_data((const char **)red_xpm);
+    = gtk_widget_render_icon(dummy, OCHUSHA_STOCK_NET_ERROR,
+			     GTK_ICON_SIZE_MENU, "ochusha::net-error");
+
+  gtk_object_sink(GTK_OBJECT(dummy));
+
   tab_label_id = g_quark_from_static_string("OchushaUI::icon-label");
   tab_label_icon_counter_id
     = g_quark_from_static_string("OchushaUI::icon-counter");
@@ -822,8 +933,9 @@ main_menu_callback(gpointer data, guint action, GtkWidget *widget)
       refresh_current_threadlist_all(application);
       return;
 
-#if 0
+#if ENABLE_DEBUG_CONSOLE
     case MAIN_MENU_DEBUG_CONSOLE:
+      open_debug_console(application);
       return;
 #endif
 
@@ -888,19 +1000,26 @@ write_optional_prefs(FILE *file, gpointer user_data)
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, board_tab_minimum_size);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  board_tab_enable_tooltips);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  board_tab_always_show);
   
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, threadlist_pane_style);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, thread_tab_shrinkable);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, thread_tab_minimum_size);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  thread_tab_enable_tooltips);
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
+				  thread_tab_always_show);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application,
 				  show_threadlist_entry_when_thread_selected);
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, threadlist_view_contents);
 
   OUTPUT_CONFIG_ATTRIBUTE_STRING(file, application, thread_view_font_name);
+
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, default_open_in_tab);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, show_mailto_mode);
 
+
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, enable_popup_title);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, popup_title_delay);
 
@@ -1116,6 +1235,8 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     application->board_tab_enable_tooltips
       = OCHUSHA_DEFAULT_BOARD_TAB_ENABLE_TOOLTIPS;
 
+  application->board_tab_always_show
+    = ochusha_utils_get_attribute_boolean(pref_attrs, "board_tab_always_show");
 
   if (g_hash_table_lookup(pref_attrs, "threadlist_pane_style") != NULL)
     application->threadlist_pane_style
@@ -1145,6 +1266,10 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     application->thread_tab_enable_tooltips
       = OCHUSHA_DEFAULT_THREAD_TAB_ENABLE_TOOLTIPS;
 
+  application->thread_tab_always_show
+    = ochusha_utils_get_attribute_boolean(pref_attrs,
+					  "thread_tab_always_show");
+
   if (g_hash_table_lookup(pref_attrs,
 			  "show_threadlist_entry_when_thread_selected")
       != NULL)
@@ -1166,6 +1291,9 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
   application->thread_view_font_name
     = ochusha_utils_get_attribute_string(pref_attrs, "thread_view_font_name");
 
+  application->default_open_in_tab
+    = ochusha_utils_get_attribute_boolean(pref_attrs, "default_open_in_tab");
+
   application->show_mailto_mode
     = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(pref_attrs,
 							   "show_mailto_mode");
@@ -1388,6 +1516,11 @@ output_log_cb(OchushaNetworkBroker *broker, const gchar *message,
 #if 0
   fprintf(stderr, "LOG: %s", message);
 #endif
+#if ENABLE_DEBUG_CONSOLE
+  g_return_if_fail(GTK_IS_TEXT_BUFFER(log_buffer));
+
+  gtk_text_buffer_insert(log_buffer, &log_tail_iter, message, -1);
+#endif
 }
 
 
@@ -1427,6 +1560,19 @@ preference_dialog_response_cb(GtkWidget *widget, gint response_id,
 				application->board_tab_enable_tooltips);
 	  }
 
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "board_tab_always_show");
+	if (tmp_data != NULL)
+	  {
+	    gboolean always_show
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	    application->board_tab_always_show = always_show;
+	    paned_notebook_set_tab_policy(boardlist_pane,
+					  always_show
+					  ? GTK_POLICY_ALWAYS
+					  : GTK_POLICY_AUTOMATIC);
+	  }
+
 #if 0	/* ɤ */
 	tmp_data = g_object_get_data(G_OBJECT(widget),
 				     "boardlist_pane_style");
@@ -1523,6 +1669,20 @@ preference_dialog_response_cb(GtkWidget *widget, gint response_id,
 				application->thread_tab_enable_tooltips);
 	  }
 
+	tmp_data = g_object_get_data(G_OBJECT(widget),
+				     "thread_tab_always_show");
+	if (tmp_data != NULL)
+	  {
+	    gboolean always_show
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	    application->thread_tab_always_show = always_show;
+	    if (current_board_pane != NULL)
+	      paned_notebook_set_tab_policy(PANED_NOTEBOOK(current_board_pane),
+					    always_show
+					    ? GTK_POLICY_ALWAYS
+					    : GTK_POLICY_AUTOMATIC);
+	  }
+
 	tmp_data = g_object_get_data(G_OBJECT(widget),
 				     "show_threadlist_entry");
 	if (tmp_data != NULL)
@@ -1577,6 +1737,28 @@ preference_dialog_response_cb(GtkWidget *widget, gint response_id,
 		 : THREAD_VIEW_MAILTO_MODE_HIDE);
 	  }
 
+	tmp_data = g_object_get_data(G_OBJECT(widget), "default_open_in_tab");
+	if (tmp_data != NULL)
+	  {
+	    application->default_open_in_tab
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	    boardlist_view_set_default_open_in_tab(
+				BOARDLIST_VIEW(application->boardlist_view),
+				application->default_open_in_tab);
+	    if (current_board_pane != NULL)
+	      {
+		GtkWidget *tmp_widget
+		  = paned_notebook_get_selector(PANED_NOTEBOOK(current_board_pane));
+		if (tmp_widget != NULL)
+		  {
+		    tmp_widget = gtk_bin_get_child(GTK_BIN(tmp_widget));
+		    threadlist_view_set_default_open_in_tab(
+					THREADLIST_VIEW(tmp_widget),
+					application->default_open_in_tab);
+		  }
+	      }
+	  }
+
 	break;
       }
 
@@ -1631,6 +1813,13 @@ build_ochusha_setting_frame(OchushaApplication *application,
   gtk_widget_show(outer_vbox);
   gtk_container_add(GTK_CONTAINER(ochusha_frame), outer_vbox);
 
+  tmp_widget = gtk_check_button_new_with_label(_("Open in Tab by Default."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->default_open_in_tab);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "default_open_in_tab", tmp_widget);
+
   tmp_widget = gtk_check_button_new_with_label(_("Enable Tooltips of Board Tab Label."));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
 			       application->board_tab_enable_tooltips);
@@ -1639,6 +1828,13 @@ build_ochusha_setting_frame(OchushaApplication *application,
   g_object_set_data(G_OBJECT(dialog), "board_tab_enable_tooltips",
 		    tmp_widget);
 
+  tmp_widget = gtk_check_button_new_with_label(_("Always Show Board Tab."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->board_tab_always_show);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 5);
+  g_object_set_data(G_OBJECT(dialog), "board_tab_always_show", tmp_widget);
+
 #if 0
   tmp_widget = gtk_check_button_new_with_label(_("Place Boardlist and Threadlist Vertically."));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
@@ -1664,9 +1860,29 @@ build_ochusha_setting_frame(OchushaApplication *application,
   gtk_widget_show(proxy_vbox);
   gtk_container_add(GTK_CONTAINER(proxy_frame), proxy_vbox);
 
+  hbox = gtk_hbox_new(FALSE, 0);
   tmp_widget = gtk_label_new(_("WARNING: Use of PROXY never tested.  If you know whether this works well or not, please report that."));
+  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
   gtk_widget_show(tmp_widget);
-  gtk_box_pack_start(GTK_BOX(proxy_vbox), tmp_widget, FALSE, FALSE, 5);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 5);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  tmp_widget = gtk_label_new(_("WARNING: If specified PROXY server isn't alive, Ochusha may hang up."));
+  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 5);
+
+  hbox = gtk_hbox_new(FALSE, 0);
+  tmp_widget = gtk_label_new(_("WARNING: Sorry, I cannot recommend to use PROXY at this time."));
+  gtk_label_set_justify(GTK_LABEL(tmp_widget), GTK_JUSTIFY_LEFT);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 5);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(proxy_vbox), hbox, FALSE, FALSE, 5);
 
   hbox = gtk_hbox_new(FALSE, 0);
 
@@ -1796,6 +2012,13 @@ build_threadlist_view_setting_frame(OchushaApplication *application,
   g_object_set_data(G_OBJECT(dialog), "thread_tab_enable_tooltips",
 		    tmp_widget);
 
+  tmp_widget = gtk_check_button_new_with_label(_("Always Show Thread Tab."));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       application->thread_tab_always_show);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "thread_tab_always_show", tmp_widget);
+
   tmp_widget = gtk_check_button_new_with_label(_("Show Threadlist Entry for Selected Thread."));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
 			       application->show_threadlist_entry_when_thread_selected);
@@ -1914,6 +2137,58 @@ build_thread_view_setting_frame(OchushaApplication *application,
 }
 
 
+#if ENABLE_DEBUG_CONSOLE
+static void
+debug_console_response_cb(GtkWidget *widget, gint response_id,
+			  gpointer unused)
+{
+  if (widget == NULL)
+    return;
+
+  gtk_widget_hide(widget);
+}
+
+
+static void
+open_debug_console(OchushaApplication *application)
+{
+  GtkWidget *dialog;
+  GtkWidget *scroll;
+  GtkWidget *text_view;
+  if (debug_console != NULL)
+    {
+      gtk_widget_show_all(debug_console);
+      return;
+    }
+
+  dialog = gtk_dialog_new_with_buttons(_("Debug Outputs"),
+				       application->top_level,
+				       GTK_DIALOG_DESTROY_WITH_PARENT,
+				       GTK_STOCK_OK, GTK_RESPONSE_OK,
+				       NULL);
+  g_signal_connect(G_OBJECT(dialog), "response",
+		   G_CALLBACK(debug_console_response_cb), application);
+  g_signal_connect(G_OBJECT(dialog), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &debug_console);
+  gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+
+  scroll = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
+				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroll),
+				      GTK_SHADOW_ETCHED_IN);
+  text_view = gtk_text_view_new_with_buffer(log_buffer);
+  gtk_widget_show(text_view);
+  gtk_container_add(GTK_CONTAINER(scroll), text_view);
+  gtk_widget_show(scroll);
+  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), scroll);
+
+  gtk_widget_show(dialog);
+  debug_console = dialog;
+}
+#endif
+
+
 static void
 open_preference_dialog(OchushaApplication *application)
 {
@@ -1974,57 +2249,57 @@ static void
 open_about_dialog(OchushaApplication *application)
 {
   MAIN_UI_LOCK
-    {
-      if (about_dialog == NULL)
-	{
-	  gchar text_buffer[1024];
-	  GtkWidget *label;
-	  GdkPixbuf *banner = NULL;
+  {
+    if (about_dialog == NULL)
+      {
+	gchar text_buffer[1024];
+	GtkWidget *label;
+	GdkPixbuf *banner = NULL;
 
-	  about_dialog = gtk_dialog_new_with_buttons(_("About Ochusha"),
+	about_dialog = gtk_dialog_new_with_buttons(_("About Ochusha"),
 					application->top_level,
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_STOCK_OK, GTK_RESPONSE_OK,
 					NULL);
-	  gtk_dialog_set_default_response(GTK_DIALOG(about_dialog),
-					  GTK_RESPONSE_OK);
-	  g_signal_connect(G_OBJECT(about_dialog), "response",
-			   G_CALLBACK(about_dialog_response_cb),
-			   NULL);
-	  g_signal_connect(G_OBJECT(about_dialog), "destroy",
-			   G_CALLBACK(gtk_widget_destroyed),
-			   &about_dialog);
+	gtk_dialog_set_default_response(GTK_DIALOG(about_dialog),
+					GTK_RESPONSE_OK);
+	g_signal_connect(G_OBJECT(about_dialog), "response",
+			 G_CALLBACK(about_dialog_response_cb),
+			 NULL);
+	g_signal_connect(G_OBJECT(about_dialog), "destroy",
+			 G_CALLBACK(gtk_widget_destroyed),
+			 &about_dialog);
 
-	  snprintf(text_buffer, 1024, "%s %s %s",
-		   _("ochusha"), _("Version"), PACKAGE_VERSION);
-	  label = gtk_label_new(text_buffer);
-	  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
-			    label);
+	snprintf(text_buffer, 1024, "%s %s %s",
+		 _("ochusha"), _("Version"), PACKAGE_VERSION);
+	label = gtk_label_new(text_buffer);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
+			  label);
 
 #ifdef HAVE_LOCALTIME_R
-	  {
-	    struct tm tm;
-	    time_t utc = time(NULL);
-	    localtime_r(&utc, &tm);
+	{
+	  struct tm tm;
+	  time_t utc = time(NULL);
+	  localtime_r(&utc, &tm);
 
-	    if (tm.tm_year == 104 && tm.tm_yday < 7)
-	      banner = banner1_pixbuf;
-	    else
-	      banner = banner_pixbuf;
-	  }
-#else
-	  banner = banner_pixbuf;
-#endif
-	  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
-			    gtk_image_new_from_pixbuf(banner));
-
-	  label = gtk_label_new("Copyright(c) 2003 The Ochusha Project");
-	  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
-			    label);
-
-	  gtk_widget_show_all(about_dialog);
+	  if (tm.tm_year == 104 && tm.tm_yday < 7)
+	    banner = banner1_pixbuf;
+	  else
+	    banner = banner_pixbuf;
 	}
-    }
+#else
+	banner = banner_pixbuf;
+#endif
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
+			  gtk_image_new_from_pixbuf(banner));
+
+	label = gtk_label_new("Copyright(c) 2003 The Ochusha Project");
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox),
+			  label);
+
+	gtk_widget_show_all(about_dialog);
+      }
+  }
   MAIN_UI_UNLOCK;
 }
 
@@ -2053,6 +2328,9 @@ initialize_open_url_dialog(OchushaApplication *application)
   g_signal_connect(G_OBJECT(open_url_dialog), "response",
 		   G_CALLBACK(open_url_dialog_response_cb),
 		   application);
+  g_signal_connect(G_OBJECT(open_url_dialog), "destroy",
+		   G_CALLBACK(gtk_widget_destroyed), &open_url_dialog);
+
 
   open_url_combo = gtk_combo_new();
   if (open_url_history != NULL)
@@ -2095,27 +2373,21 @@ open_url_dialog_response_cb(GtkWidget *dialog, gint response_id,
 
   gtk_widget_hide_all(open_url_dialog);
 
+#if 0	/* ZZZ */
   if (response_id == GTK_RESPONSE_DELETE_EVENT)
     {
-      MAIN_UI_LOCK
-	{
-	  open_url_dialog = NULL;
-	}
-      MAIN_UI_UNLOCK;
+      open_url_dialog = NULL;
     }
+#endif
 }
 
 
 static void
 popup_open_url_dialog(OchushaApplication *application)
 {
-  MAIN_UI_LOCK
-    {
-      if (open_url_dialog == NULL)
-	initialize_open_url_dialog(application);
-      gtk_widget_show_all(open_url_dialog);
-    }
-  MAIN_UI_UNLOCK;
+  if (open_url_dialog == NULL)
+    initialize_open_url_dialog(application);
+  gtk_widget_show_all(open_url_dialog);
 }
 
 
