@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.h,v 1.23 2003/11/25 18:19:23 fuyu Exp $
+ * $Id: ochusha_ui.h,v 1.27 2003/12/03 12:01:10 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_UI_H_
@@ -52,6 +52,7 @@
 #define DEBUG_WIDGET_MOST	0
 #define DEBUG_RENDERING_ON_IDLE	0
 
+#define ENABLE_DEBUG_CONSOLE						0
 #define ENABLE_MAIN_TOOLBAR						0
 #define ENABLE_RENDERING_ON_IDLE					0
 
@@ -102,6 +103,16 @@
 
 #define OCHUSHA_MINIMIZED_ICON			"Ochusha-Minimized"
 
+#define OCHUSHA_STOCK_WRITE_RESPONSE		"Ochusha-Write-Response"
+#define OCHUSHA_STOCK_NET_PROGRESS0		"Ochusha-Net-Progress-0"
+#define OCHUSHA_STOCK_NET_PROGRESS1		"Ochusha-Net-Progress-1"
+#define OCHUSHA_STOCK_NET_PROGRESS2		"Ochusha-Net-Progress-2"
+#define OCHUSHA_STOCK_NET_PROGRESS3		"Ochusha-Net-Progress-3"
+#define OCHUSHA_STOCK_NET_PREPARATION		"Ochusha-Net-Preparation"
+#define OCHUSHA_STOCK_NET_DONE			"Ochusha-Net-Done"
+#define OCHUSHA_STOCK_NET_ERROR			"Ochusha-Net-Error"
+
+
 #define TAB_LABEL_ICON_PROGRESS0		0
 #define TAB_LABEL_ICON_PROGRESS1		1
 #define TAB_LABEL_ICON_PROGRESS2		2
@@ -186,12 +197,15 @@ struct _OchushaApplication
   gboolean board_tab_shrinkable;
   guint board_tab_minimum_size;
   gboolean board_tab_enable_tooltips;
+  gboolean board_tab_always_show;
   gboolean hide_hidden_boardlist_items;
 
+
   gint threadlist_pane_style;
   gboolean thread_tab_shrinkable;
   guint thread_tab_minimum_size;
   gboolean thread_tab_enable_tooltips;
+  gboolean thread_tab_always_show;
   gboolean show_threadlist_entry_when_thread_selected;
   char *threadlist_view_contents;
 
@@ -200,7 +214,7 @@ struct _OchushaApplication
   char *thread_view_font_name;
 
   /* ΤΥǥե */
-  /* ¾ĤΥǥեȤȥ̤Ѱդͽꡣ */
+  gboolean default_open_in_tab;
   ThreadViewMailtoMode show_mailto_mode;
 
 
