@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.7 2003/11/26 04:13:51 fuyu Exp $
+ * $Id: htmlutils.c,v 1.8 2003/11/26 12:59:36 fuyu Exp $
  */
 
 #include "config.h"
@@ -921,8 +921,11 @@ process_entity_ref(ElementHandler *handler, void *context, const char *head,
       {
 	gchar *utf8_entity = NULL;
 	gchar broken_ref_buffer[DEFAULT_BUFFER_SIZE];
-	utf8_entity = convert_string(handler->converter, handler->helper,
-				     entity, -1);
+	if (handler->converter != NULL)
+	  utf8_entity = convert_string(handler->converter, handler->helper,
+				       entity, -1);
+	else
+	  utf8_entity = entity;
 	if (utf8_entity != NULL)
 	  snprintf(broken_ref_buffer, DEFAULT_BUFFER_SIZE, "&%s%s",
 		   utf8_entity, (*ref_tail == ';' ? ";" : ""));
@@ -931,7 +934,7 @@ process_entity_ref(ElementHandler *handler, void *context, const char *head,
 		   entity, (*ref_tail == ';' ? ";" : ""));
 	(*handler->characters)(context, broken_ref_buffer,
 			       strlen(broken_ref_buffer));
-	if (utf8_entity != NULL)
+	if (utf8_entity != NULL && utf8_entity != entity)
 	  G_FREE(utf8_entity);
 #if DEBUG_PARSER_MOST
 	fprintf(stderr, "broken entity ref: %s -> %s\n", entity, broken_ref_buffer);
