@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.5 2003/06/07 14:35:31 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.10 2003/12/04 08:50:46 fuyu Exp $
  */
 
 #include "config.h"
@@ -31,6 +31,7 @@
 #include "ochusha_private.h"
 
 #include "ochusha_async_buffer.h"
+#include "marshal.h"
 
 #include <glib.h>
 
@@ -200,6 +201,15 @@ static GSList *ochusha_async_buffer_list = NULL;
     }
 
 
+enum {
+  WAKEUP_NOW_SIGNAL,
+  LAST_SIGNAL
+};
+
+
+static gint async_buffer_signals[LAST_SIGNAL] = { 0, };
+
+
 static void
 ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass)
 {
@@ -218,6 +228,16 @@ ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass)
     }
 
   parent_class = g_type_class_peek_parent(klass);
+
+  async_buffer_signals[WAKEUP_NOW_SIGNAL] =
+    g_signal_new("wakeup_now",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(OchushaAsyncBufferClass, wakeup_now),
+		 NULL, NULL,
+		 libochusha_marshal_VOID__VOID,
+		 G_TYPE_NONE, 0);
+
   o_class->finalize = ochusha_async_buffer_finalize;
 }
 
@@ -439,10 +459,13 @@ ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
   LOCK_BUFFER_LOCK(buffer);
   buffer->number_of_suspended_users = 0;
   buffer->state = OCHUSHA_ASYNC_BUFFER_SUSPENDED;
-
+  BROADCAST_BUFFER_CONDITION(buffer);
   LOCK_GLOBAL_LOCK;
 
-  BROADCAST_BUFFER_CONDITION(buffer);
+  g_signal_emit(G_OBJECT(buffer),
+		async_buffer_signals[WAKEUP_NOW_SIGNAL],
+		0);
+
   UNLOCK_BUFFER_LOCK(buffer);		/* νǤʤȤʤʤ
 					 * UNLOCK_BUFFER_LOCKִ֤
 					 * ΥåԤäƤåɤ
@@ -455,6 +478,7 @@ ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 					 * 뤿ΤΤǤ
 					 * ա
 					 */
+
   while (buffer->number_of_suspended_users < buffer->number_of_active_users)
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
@@ -510,6 +534,10 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
 	  "ochusha_async_buffer_terminate: global lock locked\n");
 #endif
 
+  g_signal_emit(G_OBJECT(buffer),
+		async_buffer_signals[WAKEUP_NOW_SIGNAL],
+		0);
+
   UNLOCK_BUFFER_LOCK(buffer);
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
