@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.20 2003/11/25 18:20:44 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.21 2003/11/29 08:52:05 fuyu Exp $
  */
 
 #include "config.h"
@@ -40,29 +40,6 @@
 #include <glib-object.h>
 #include <glib.h>
 
-#if ENABLE_REGEXP
-# if HAVE_ONIGURUMA
-#  if HAVE_ONIG_ONIGPOSIX_H
-#    include <onig/onigposix.h>
-#  elif HAVE_ONIGPOSIX_H
-#    include <onigposix.h>
-#  else
-#    error "Use of oniguruma without onig/onigposix.h header isn't considered."
-#  endif
-# else
-#  if HAVE_POSIX_REGEX
-#    if HAVE_SYS_TYPES_H
-#      include <sys/types.h>
-#    endif
-#    if HAVE_REGEX_H
-#      include <regex.h>
-#    else
-#      warning "I don't know suitable header file of your posix regex library."
-#    endif
-#  endif
-# endif
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -86,8 +63,6 @@ static void ochusha_board_2ch_write_boardlist_element(
 
 static const char *ochusha_board_2ch_get_subject_txt_encoding(
 						OchushaBulletinBoard *board);
-static const char *ochusha_board_2ch_get_subback_html_encoding(
-						OchushaBulletinBoard *board);
 
 static char *ochusha_board_2ch_generate_base_path(OchushaBulletinBoard *board,
 						  const char *url);
@@ -280,34 +255,9 @@ ochusha_board_2ch_get_subject_txt_encoding(OchushaBulletinBoard *board)
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_MITINOKU:
       return "CP932";
 
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      return "EUC-JP";
-
-    default:
-      return NULL;
-    }
-}
-
-
-static const char *
-ochusha_board_2ch_get_subback_html_encoding(OchushaBulletinBoard *board)
-{
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      return "CP932";
-
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
       return "EUC-JP";
 
     default:
@@ -320,38 +270,7 @@ static char *
 ochusha_board_2ch_generate_base_path(OchushaBulletinBoard *board,
 				     const char *url)
 {
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      {
-	char *path;
-	char *abs_path = ochusha_utils_url_extract_http_absolute_path(url);
-	char *tmp_pos;
-	g_return_val_if_fail(abs_path != NULL, NULL);
-
-	if (abs_path[0] != '/')
-	  {
-	    G_FREE(abs_path);
-	    return NULL;
-	  }
-	tmp_pos = strchr(abs_path + 1, '/');
-
-	if (tmp_pos != NULL)
-	  {
-	    tmp_pos[1] = '\0';
-	    path = G_STRDUP(abs_path);
-	  }
-	else
-	  {
-	    path = G_STRDUP("/");
-	  }
-	G_FREE(abs_path);
-	return path;
-      }
-
-    default:
-      return G_STRDUP("/");
-    }
+  return G_STRDUP("/");
 }
 
 
@@ -372,8 +291,6 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
     {
     case OCHUSHA_BBS_TYPE_2CH:
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_MITINOKU:
       {
 	char *tmp_pos = strchr(abs_path + 1, '/');
 	if (tmp_pos != NULL)
@@ -384,21 +301,6 @@ ochusha_board_2ch_generate_board_id(OchushaBulletinBoard *board,
 	break;
       }
 
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      {
-	char *tmp_pos = strchr(abs_path + 1, '/');
-	if (tmp_pos == NULL)
-	  id = G_STRDUP(abs_path + 1);
-	else
-	  {
-	    char *id_tail = strchr(tmp_pos + 1, '/');
-	    if (id_tail != NULL)
-	      *id_tail = '\0';
-	    id = G_STRDUP(tmp_pos + 1);
-	  }
-	break;
-      }
-
     default:
       id = NULL;	/* ̤ݡ */
       break;
@@ -459,13 +361,9 @@ ochusha_board_2ch_get_response_character_encoding(OchushaBulletinBoard *board)
   switch (((OchushaBulletinBoard *)board)->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
       return "CP932";
 
     case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-    case OCHUSHA_BBS_TYPE_MITINOKU:
       return "EUC-JP";
 
     default:
@@ -482,11 +380,8 @@ ochusha_board_2ch_get_response_iconv_helper(OchushaBulletinBoard *board)
   switch (((OchushaBulletinBoard *)board)->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
       return cp932_to_utf8_helper;
 
-    case OCHUSHA_BBS_TYPE_MITINOKU:
     default:
       return NULL;
     }
@@ -538,7 +433,6 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
     case OCHUSHA_BBS_TYPE_2CH:
       break;
 
-      /* case OCHUSHA_BBS_TYPE_MACHIBBS: */
     default:
       return FALSE;	/* ̤ݡ */
     }
@@ -595,11 +489,6 @@ ochusha_board_2ch_create_new_thread(OchushaBulletinBoard *board,
       query = g_strdup_printf("submit=%%90%%56%%8B%%4B%%83%%58%%83%%8C%%83%%62%%83%%68%%8D%%EC%%90%%AC&subject=%s&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&time=%ld", title, from, mail, message, bbs, time);
       break;
 
-#if 0
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      query = g_strdup_printf("submit=%%90%%56%%8B%%4B%%83%%58%%83%%8C%%83%%62%%83%%68%%8D%%EC%%90%%AC&NAME=%s&MAIL=%s&MESSAGE=%s&BBS=%s&KEY=%s&TIME=%ld", from, mail, message, bbs, key, time);
-      break;
-#endif
     default:
       goto error_exit;
     }
@@ -685,7 +574,7 @@ ochusha_board_2ch_get_read_cgi_url(OchushaBoard2ch *board_2ch)
 
   klass = OCHUSHA_BOARD_2CH_GET_CLASS(board_2ch);
   if (klass->get_read_cgi_url != NULL)
-    board_2ch->read_cgi_url = (klass->get_read_cgi_url)(board_2ch);
+    board_2ch->read_cgi_url = (*klass->get_read_cgi_url)(board_2ch);
   else
     {
       OchushaBulletinBoard *board = OCHUSHA_BULLETIN_BOARD(board_2ch);
@@ -700,31 +589,6 @@ ochusha_board_2ch_get_read_cgi_url(OchushaBoard2ch *board_2ch)
 	    board_2ch->read_cgi_url = G_STRDUP(url);
 	  break;
 
-	case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-	  if (snprintf(url, PATH_MAX, "http://%s%sbbs/read.cgi",
-		       ochusha_bulletin_board_get_server(board),
-		       ochusha_bulletin_board_get_base_path(board)) < PATH_MAX)
-	    board_2ch->read_cgi_url = G_STRDUP(url);
-	  break;
-
-	case OCHUSHA_BBS_TYPE_JBBS:
-	  if (snprintf(url, PATH_MAX, "http://%s/bbs/read.cgi",
-		       ochusha_bulletin_board_get_server(board)) < PATH_MAX)
-	    board_2ch->read_cgi_url = G_STRDUP(url);
-	  break;
-
-	case OCHUSHA_BBS_TYPE_MACHIBBS:
-	  if (snprintf(url, PATH_MAX, "http://%s/bbs/read.pl",
-		       ochusha_bulletin_board_get_server(board)) < PATH_MAX)
-	    board_2ch->read_cgi_url = G_STRDUP(url);
-	  break;
-
-	case OCHUSHA_BBS_TYPE_MITINOKU:
-	  if (snprintf(url, PATH_MAX, "http://%s/read.cgi",
-		       ochusha_bulletin_board_get_server(board)) < PATH_MAX)
-	    board_2ch->read_cgi_url = G_STRDUP(url);
-	  break;
-
 	default:
 	  return NULL;
 	}
@@ -764,27 +628,11 @@ ochusha_board_2ch_get_threadlist_source(OchushaBulletinBoard *board,
 
   g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(board), NULL);
 
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      if (snprintf(url, PATH_MAX, "%s" OCHUSHA_SUBBACK_HTML, board->base_url)
-	  >= PATH_MAX)
-	return NULL;
-      return ochusha_network_broker_read_from_cgi(broker, url);
-
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-    default:
-      if (snprintf(url, PATH_MAX, "%s" OCHUSHA_SUBJECT_TXT, board->base_url)
-	  >= PATH_MAX)
-	return NULL;
-    }
+  if (snprintf(url, PATH_MAX, "%s" OCHUSHA_SUBJECT_TXT, board->base_url)
+      >= PATH_MAX)
+    return NULL;
 
   board_2ch = OCHUSHA_BOARD_2CH(board);
-
   return ochusha_network_broker_read_from_url(broker, url,
 					      board_2ch->last_modified, mode);
 }
@@ -891,12 +739,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
   switch (board->bbs_type)
     {
     case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_JBBS:
       converter_helper = cp932_to_utf8_helper;
       break;
 
-    case OCHUSHA_BBS_TYPE_MITINOKU:
     default:
       converter_helper = NULL;
     }
@@ -934,28 +779,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    int n_responses_on_server = 0;
 	    char *tmp_pos;
 	    int title_len;
+	    char *ext;
 
-	    /*
-	     * XXX: 롼פǤʤȤΤϤ礤»
-	     */
-	    switch (board->bbs_type)
-	      {
-	      case OCHUSHA_BBS_TYPE_2CH:
-	      case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-	      case OCHUSHA_BBS_TYPE_MITINOKU:
-		tmp_pos = g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
-		break;
-
-	      case OCHUSHA_BBS_TYPE_JBBS:
-	      case OCHUSHA_BBS_TYPE_MACHIBBS:
-	      case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-		tmp_pos = memchr(cur_pos, ',', eol_pos - cur_pos);
-		break;
-
-	      default:
-		tmp_pos = NULL;
-		break;
-	      }
+	    tmp_pos = g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
 
 	    if (tmp_pos == NULL)
 	      {
@@ -972,37 +798,9 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 	    else
 	      thread_id = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
-	    /*
-	     * XXX: 롼פǤʤȤΤϤ礤»
-	     */
-	    switch (board->bbs_type)
-	      {
-	      case OCHUSHA_BBS_TYPE_2CH:
-	      case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-	      case OCHUSHA_BBS_TYPE_MITINOKU:
-		{
-		  char *ext;
-		  if ((ext = strstr(thread_id, ".dat")) != NULL)
-		    *ext = '\0';
-		  title_pos = tmp_pos + 2;	/* skip "<>" */
-		  break;
-		}
-
-	      case OCHUSHA_BBS_TYPE_JBBS:
-	      case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-	      case OCHUSHA_BBS_TYPE_MACHIBBS:
-		{
-		  char *ext;
-		  if ((ext = strstr(thread_id, ".cgi")) != NULL)
-		    *ext = '\0';
-		  title_pos = tmp_pos + 1;	/* skip "," */
-		  break;
-		}
-
-	      default:
-		title_pos = NULL;
-		abort();
-	      }
+	    if ((ext = strstr(thread_id, ".dat")) != NULL)
+	      *ext = '\0';
+	    title_pos = tmp_pos + 2;	/* skip "<>" */
 
 	    title_len = eol_pos - title_pos;
 
@@ -1141,233 +939,6 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 }
 
 
-#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
-/*
- * refresh_threadlist_by_subback_html:
- *
- * Ĥsubback.htmlȸʤƲϤͿ줿
- * OchushaBulletinBoard¤Τ򹹿롣
- * ĸĤ٤˻ꤵ줿ХåؿƤ֡
- *
- * ХåؿFALSE֤硢λǲϤλ롣
- *
- * ϤˤTRUE֤ԤFALSE֤
- */
-static gboolean
-refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
-				   OchushaAsyncBuffer *buffer,
-				   EachThreadCallback *each_thread_cb,
-				   gpointer callback_data)
-{
-  gboolean result;
-  GSList *thread_list = NULL;
-  GSList *old_thread_list = board->thread_list;
-  iconv_t converter;
-  iconv_helper *converter_helper;
-  regex_t threadlist_entry_pattern;
-  OchushaNetworkBrokerBufferStatus *status
-    = g_object_get_data(G_OBJECT(buffer),
-			"OchushaNetworkBroker::BufferStatus");
-
-  g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(board) && status != NULL, FALSE);
-
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      converter_helper = NULL;
-      if (regcomp(&threadlist_entry_pattern,
-		  ".*</a>:<a href=.*key=([1-9][0-9]*)[^>]*>(.*)&lt;([1-9][0-9]*)&gt;</a><br>$", REG_EXTENDED))
-	{
-	  fprintf(stderr, "invalid regular expression\n");
-	  return FALSE;
-	}
-      break;
-
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_JBBS:
-    default:
-      converter_helper = cp932_to_utf8_helper;
-      return FALSE;	/* ̤ݡ */
-    }
-
-  converter = iconv_open("UTF-8//IGNORE",
-			 ochusha_board_2ch_get_subback_html_encoding(board));
-  if (converter == (iconv_t)-1)
-    regfree(&threadlist_entry_pattern);
-  g_return_val_if_fail(converter != (iconv_t)-1, FALSE);
-
-  if (!ochusha_async_buffer_active_ref(buffer, "ochusha_board_2ch.c: ochusha_board_2ch_refresh_threadlist"))
-    {
-#if DEBUG_ASYNC_BUFFER_MOST
-      fprintf(stderr, "buffer has been terminated.\n");
-#endif
-      iconv_close(converter);
-      regfree(&threadlist_entry_pattern);
-      return FALSE;
-    }
-
-  g_slist_foreach(old_thread_list, unmark_alive, NULL);
-
-  ochusha_async_buffer_lock(buffer);
-  {
-    unsigned int offset = 0;
-
-    while (TRUE)
-      {
-	char *buffer_top = (char *)buffer->buffer;
-	char *cur_pos = buffer_top + offset;
-	unsigned int rest_of_data = buffer->length - offset;
-	char *eol_pos;
-
-	while (rest_of_data > 0
-	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
-	  {
-	    char *thread_id;
-	    gchar *thread_title;
-	    OchushaBBSThread *thread;
-	    OchushaThread2ch *thread_2ch;
-	    gchar *html_line;
-	    int n_responses_on_server = 0;
-	    regmatch_t match[4];
-
-	    *eol_pos = '\0';
-	    html_line = convert_string(converter, converter_helper,
-				       cur_pos, -1);
-	    *eol_pos = '\n';	/* ̣ʤ̣ġ */
-
-	    if (html_line == NULL
-		|| regexec(&threadlist_entry_pattern,
-			   html_line, 3, match, 0) != 0)
-	      {
-#if 0
-		fprintf(stderr, "html_line: \"%s\"\n", html_line);
-#endif
-		goto prepare_next_line;
-	      }
-
-	    thread_id = html_line + match[1].rm_so;
-	    thread_id[match[1].rm_eo - match[1].rm_so] = '\0';
-
-	    thread = ochusha_bulletin_board_lookup_bbs_thread_by_id(board,
-								    thread_id);
-	    if (thread == NULL)
-	      {
-		thread_title
-		  = simple_string_canon(html_line + match[2].rm_so,
-					match[2].rm_eo - match[2].rm_so,
-					NULL, NULL);
-		thread = ochusha_bulletin_board_bbs_thread_new(board,
-							       thread_id,
-							       thread_title);
-		G_FREE(thread_title);
-	      }
-
-	    html_line[match[3].rm_eo] = '\0';
-	    sscanf(html_line + match[3].rm_so, "%d", &n_responses_on_server);
-
-	    thread_2ch = OCHUSHA_THREAD_2CH(thread);
-
-	    if (!thread_2ch->alive)
-	      {
-		thread_2ch->alive = TRUE;
-
-		thread->flags &= ~OCHUSHA_BBS_THREAD_DAT_DROPPED;
-		thread->number_of_responses_on_server = n_responses_on_server;
-
-		g_object_ref(G_OBJECT(thread));
-		thread_list = g_slist_append(thread_list, thread);
-
-		if (each_thread_cb != NULL
-		    && !(*each_thread_cb)(thread, callback_data))
-		  {
-		    result = FALSE;
-		    break;
-		  }
-	      }
-
-	  prepare_next_line:
-	    if (html_line != NULL)
-	      G_FREE(html_line);
-	    offset = (eol_pos + 1) - buffer_top;
-	    buffer_top = (char *)buffer->buffer;
-	    cur_pos = buffer_top + offset;
-	    rest_of_data = buffer->length - offset;
-	  }
-
-	if (buffer->fixed)
-	  {
-	    result = TRUE;
-	    break;
-	  }
-
-	if (!ochusha_async_buffer_wait(buffer, "ochusha_board_2ch.c: ochusha_board_2ch_refresh_threadlist"))
-	  {
-#if DEBUG_ASYNC_BUFFER_MOST
-	    fprintf(stderr, "buffer has been terminated.\n");
-#endif
-	    result = FALSE;
-	    break;
-	  }
-
-	if (status->state == OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
-	  {
-	    result = FALSE;
-	    break;
-	  }
-      }
-  }
-  ochusha_async_buffer_unlock(buffer);
-
-  if (result)
-    {
-      OchushaBoard2ch *board_2ch = OCHUSHA_BOARD_2CH(board);
-
-      if (status->last_modified != NULL)
-	{
-	  if (board_2ch->last_modified != NULL)
-	    G_FREE(board_2ch->last_modified);
-	  board_2ch->last_modified = G_STRDUP(status->last_modified);
-	}
-
-      if (status->date != NULL)
-	{
-	  if (board_2ch->date != NULL)
-	    G_FREE(board_2ch->date);
-	  board_2ch->date = G_STRDUP(status->date);
-	}
-    }
-
-  ochusha_async_buffer_active_unref(buffer, "ochusha_board_2ch.c: ochusha_board_2ch_refresh_threadlist");
-
-  if (result)
-    {
-      CollectDroppedThreadArgs args =
-	{
-	  thread_list,
-	  each_thread_cb,
-	  callback_data
-	};
-
-      g_slist_foreach(old_thread_list, collect_dropped_thread, &args);
-      board->thread_list = args.thread_list;
-      g_slist_free(old_thread_list);
-    }
-  else
-    {
-      /* 顼ϸŤΤͥ */
-      g_slist_foreach(old_thread_list, undo_thread_ref, NULL);
-      g_slist_free(thread_list);
-    }
-
-  regfree(&threadlist_entry_pattern);
-  iconv_close(converter);
-
-  return result;
-}
-#endif
-
-
 /*
  * ochusha_board_2ch_refresh_threadlist:
  *
@@ -1385,19 +956,6 @@ ochusha_board_2ch_refresh_threadlist(OchushaBulletinBoard *board,
 				     EachThreadCallback *each_thread_cb,
 				     gpointer callback_data)
 {
-#if defined(ENABLE_REGEXP) && defined(HAVE_ONIGURUMA)
-  switch (board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      return refresh_threadlist_by_subback_html(board, buffer,
-						each_thread_cb, callback_data);
-
-    default:
-      return refresh_threadlist_by_subject_txt(board, buffer,
-					       each_thread_cb, callback_data);
-    }
-#else
   return refresh_threadlist_by_subject_txt(board, buffer,
 					   each_thread_cb, callback_data);
-#endif
 }
