@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.9 2003/11/23 21:13:44 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.10 2003/11/25 18:20:44 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -138,7 +138,8 @@ struct _OchushaBulletinBoardClass
 				 gpointer callback_data);
   gboolean (*new_thread_supported)(OchushaBulletinBoard *board);
   gboolean (*create_new_thread)(OchushaBulletinBoard *board,
-				OchushaConfig *config, const gchar *title,
+				OchushaNetworkBroker *broker,
+				const gchar *title,
 				const OchushaBBSResponse *response);
 };
 
@@ -198,7 +199,7 @@ gboolean ochusha_bulletin_board_preview_new_thread(OchushaBulletinBoard *board,
 gboolean ochusha_bulletin_board_is_new_thread_supported(
 						OchushaBulletinBoard *board);
 gboolean ochusha_bulletin_board_create_new_thread(OchushaBulletinBoard *board,
-					OchushaConfig *config,
+					OchushaNetworkBroker *broker,
 					const gchar *title,
 					const OchushaBBSResponse *response);
 
