@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.30 2003/11/26 15:05:55 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.33 2003/11/30 19:32:34 fuyu Exp $
  */
 
 #include "config.h"
@@ -33,7 +33,6 @@
 #include "ochusha_board_2ch.h"
 #include "ochusha_thread_2ch.h"
 #include "ochusha_utils_2ch.h"
-#include "ochusha_utils_jbbs.h"
 
 #include <glib-object.h>
 #include <glib.h>
@@ -48,7 +47,6 @@
 #define DEBUG_DAT	0
 #define DEBUG_DAT_MOST	0
 
-#define MAX_RESPONSE		1024
 #define ANALYSIS_INTERVAL	20
 
 
@@ -56,6 +54,7 @@ static void ochusha_thread_2ch_class_init(OchushaThread2chClass *klass);
 static void ochusha_thread_2ch_init(OchushaThread2ch *thread);
 static void ochusha_thread_2ch_finalize(GObject *object);
 
+static const char *ochusha_thread_2ch_get_base_url(OchushaBBSThread *thread);
 static void ochusha_thread_2ch_read_threadlist_element(
 						OchushaBBSThread *thread,
 						GHashTable *thread_attributes);
@@ -153,8 +152,7 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
   b_class->get_responses_source = ochusha_thread_2ch_get_responses_source;
   b_class->parse_responses = ochusha_thread_2ch_parse_responses;
   b_class->get_last_modified_utc = ochusha_thread_2ch_get_last_modified_utc;
-  b_class->get_url
-    = (const char *(*)(OchushaBBSThread *))ochusha_thread_2ch_get_base_url;
+  b_class->get_url = ochusha_thread_2ch_get_base_url;
   b_class->get_url_for_response = ochusha_thread_2ch_get_url_for_response;
   b_class->get_url_to_post = ochusha_thread_2ch_get_url_to_post;
   b_class->check_url = ochusha_thread_2ch_check_url;
@@ -163,6 +161,9 @@ ochusha_thread_2ch_class_init(OchushaThread2chClass *klass)
   b_class->preview_response = ochusha_thread_2ch_preview_response;
   b_class->post_supported = ochusha_thread_2ch_post_supported;
   b_class->post_response = ochusha_thread_2ch_post_response;
+
+  klass->get_base_path = NULL;
+  klass->make_post_response_message = NULL;
 }
 
 
@@ -405,16 +406,17 @@ ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread_2ch)
 }
 
 
-const char *
-ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread_2ch)
+static const char *
+ochusha_thread_2ch_get_base_url(OchushaBBSThread *thread)
 {
   char url[PATH_MAX];
   const char *base_path;
   const char *server;
-  OchushaBBSThread *thread;
+  OchushaThread2ch *thread_2ch;
 
-  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
+  g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), NULL);
 
+  thread_2ch = OCHUSHA_THREAD_2CH(thread);
   if (thread_2ch->base_url != NULL)
     return thread_2ch->base_url;
 
@@ -435,57 +437,28 @@ ochusha_thread_2ch_get_base_url(OchushaThread2ch *thread_2ch)
 const char *
 ochusha_thread_2ch_get_base_path(OchushaThread2ch *thread_2ch)
 {
-  char base_path[PATH_MAX];
-  OchushaBBSThread *thread;
-
+  OchushaThread2chClass *klass;
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
 
   if (thread_2ch->base_path != NULL)
     return thread_2ch->base_path;
 
-  thread = OCHUSHA_BBS_THREAD(thread_2ch);
-
-  g_return_val_if_fail(thread->board != NULL && thread->id != NULL, NULL);
-
-  switch (thread->board->bbs_type)
+  klass = OCHUSHA_THREAD_2CH_GET_CLASS(thread_2ch);
+  if (klass->get_base_path != NULL)
+    thread_2ch->base_path = (*klass->get_base_path)(thread_2ch);
+  else
     {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
+      OchushaBBSThread *thread = OCHUSHA_BBS_THREAD(thread_2ch);
+      char base_path[PATH_MAX];
+      g_return_val_if_fail(thread->board != NULL && thread->id != NULL, NULL);
+
       if (snprintf(base_path, PATH_MAX, "/test/read.cgi%s%s/%s/",
 		   ochusha_bulletin_board_get_base_path(thread->board),
 		   ochusha_bulletin_board_get_id(thread->board),
-		   thread->id) >= PATH_MAX)
-	return NULL;
-      break;
-
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      if (snprintf(base_path, PATH_MAX, "/bbs/read.pl?BBS=%s&KEY=%s",
-		   ochusha_bulletin_board_get_id(thread->board), thread->id)
-	  >= PATH_MAX)
-	return NULL;
-      break;
-
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      if (snprintf(base_path, PATH_MAX, "/bbs/read.cgi%s%s/%s/",
-		   ochusha_bulletin_board_get_base_path(thread->board),
-		   ochusha_bulletin_board_get_id(thread->board),
-		   thread->id) >= PATH_MAX)
-	return NULL;
-      break;
-
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      if (snprintf(base_path, PATH_MAX, "/read.cgi?bbs=%s&key=%s",
-		   ochusha_bulletin_board_get_id(thread->board), thread->id)
-	  >= PATH_MAX)
-	return NULL;
-      break;
-
-    case OCHUSHA_BBS_TYPE_JBBS:
-    default:	/* ̤ݡ */
-      return NULL;
+		   thread->id) < PATH_MAX)
+	thread_2ch->base_path = G_STRDUP(base_path);
     }
 
-  thread_2ch->base_path = G_STRDUP(base_path);
   return thread_2ch->base_path;
 }
 
@@ -501,31 +474,18 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
-  if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
+  if (thread->flags
+      & (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_STOPPED))
     mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
 
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_2CHLIKE_EUCJP:
-      return ochusha_network_broker_read_from_url(broker,
+  return ochusha_network_broker_read_from_url(broker,
 				ochusha_thread_2ch_get_dat_url(thread_2ch),
 				thread_2ch->last_modified, mode);
-
-    case OCHUSHA_BBS_TYPE_JBBS:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      return ochusha_utils_jbbs_fake_responses_source(thread_2ch, broker,
-						      mode);
-    default:
-      return NULL;
-    }
 }
 
 
 /*
- * extract_response
+ * extract_2ch_response
  *
  * cur_posʸ2chDATǡ1쥹ʬȸʤƲϤ
  * 1쥹ɽOchushaBBSResponse¤Τ˳Ǽ롣
@@ -537,7 +497,8 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
  * ľʸؤݥ󥿤֤ԻˤNULL֤
  */
 static char *
-extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
+extract_2ch_response(char *start_pos, char *eol_pos,
+		     OchushaBBSResponse *response)
 {
   /*
    * DATեγƹԤ
@@ -551,38 +512,32 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   int i = 0;
 
   /* DATǡƤ뤳Ȥ⤢Τǥå */
-  tmp_pos = memchr(start_pos, '\0', eol_pos - cur_pos);
+  tmp_pos = memchr(start_pos, '\0', eol_pos - start_pos);
   if (tmp_pos != NULL)
     {
       int rest;
+      int clone_len = eol_pos - start_pos;
       /* 줿DATǡ */
-      clone_buffer = G_MALLOC(eol_pos - cur_pos);
-      memcpy(clone_buffer, start_pos, eol_pos - cur_pos);
+      clone_buffer = G_MALLOC(clone_len + 1);
+      memcpy(clone_buffer, start_pos, clone_len);
+      rest = clone_len;
+
+      cur_pos = clone_buffer + (tmp_pos - start_pos);
+      eol_pos = clone_buffer + clone_len;
       /*
-       * XXX: ԡƤ'\0'ŬʸѤΤȡ
-       *      '\0'ФʤͤƥԡΤΤɤ餬ɤΤġġ
-       *      ҤȤޤԡơƬ'\0'Ф
+       * DATե1ʬ˴ޤޤƤ롢
+       * ٤Ǥʤ'\0''*'֤롣
        */
-      rest = eol_pos - cur_pos;
+      while (*cur_pos == '\0' && *cur_pos != '\n')
+	{
+	  *cur_pos = '*';
+	  cur_pos++;
+	}
 
       cur_pos = clone_buffer;
-      eol_pos = clone_buffer + rest;
-      while (*cur_pos == '\0')
-	cur_pos++;
-
-      rest -= (cur_pos - clone_buffer);
-
-      tmp_pos = cur_pos + 1;
-      while (--rest > 0)
-	if (*tmp_pos == '\0')
-	  *tmp_pos++ = '*';
-
 #if DEBUG_DAT
-      {
-	char *broken_line = G_STRNDUP(cur_pos, eol_pos - cur_pos);
-	fprintf(stderr, "broken line: %s\n", broken_line);
-	G_FREE(broken_line);
-      }
+      clone_buffer[clone_len] = '\0';
+      fprintf(stderr, "broken line: %s\n", clone_buffer);
 #endif
     }
 
@@ -619,9 +574,6 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   response->name = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
   cur_pos = tmp_pos + (comma_delimiter ? 1 : 2);
-  /* DATǡƤ뤳Ȥ⤢ */
-  while (*cur_pos == '\0')
-    cur_pos++;
 
   tmp_pos = comma_delimiter ? memchr(cur_pos, ',', eol_pos - cur_pos)
                             : g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
@@ -631,9 +583,6 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
     response->mailto = NULL;
 
   cur_pos = tmp_pos + (comma_delimiter ? 1 : 2);
-  /* DATǡƤ뤳Ȥ⤢ */
-  while (*cur_pos == '\0')
-    cur_pos++;
 
   tmp_pos = comma_delimiter ? memchr(cur_pos, ',', eol_pos - cur_pos)
                             : g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
@@ -643,9 +592,6 @@ extract_response(char *start_pos, char *eol_pos, OchushaBBSResponse *response)
   response->date_id = G_STRNDUP(cur_pos, tmp_pos - cur_pos);
 
   cur_pos = tmp_pos + (comma_delimiter ? 1 : 2);
-  /* DATǡƤ뤳Ȥ⤢ */
-  while (*cur_pos == 0)
-    cur_pos++;
 
   tmp_pos = g_strstr_len(cur_pos, eol_pos - cur_pos, "<>");
   if (tmp_pos == NULL)
@@ -777,14 +723,16 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   if (responses == NULL)
     {
       /*
-       * 1000쥹ʬǤ⡹4000ХȤʤΤǸĹ϶Ȥޤ衣
+       * 1000쥹ʬǤ⡹4000ХȤʤΤǡ
+       * ǽ餫MAX_RESPONSE쥹ʬ϶Ȥ롣
        */
       responses = G_NEW0(unsigned int, MAX_RESPONSE);
       thread_2ch->responses = responses;
+      thread_2ch->responses_length = MAX_RESPONSE;
     }
 
-  if (number == -1 || number > MAX_RESPONSE)
-    number = MAX_RESPONSE;
+  if (number < -1)
+    number = thread_2ch->responses_length;
 
   ochusha_async_buffer_lock(buffer);
   {
@@ -799,7 +747,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
     i = 0;
     if (status->state != OCHUSHA_NETWORK_BROKER_BUFFER_STATE_CACHE_IS_DIRTY)
       {
-	if (start > 1 && start < MAX_RESPONSE)
+	if (start > 1 && start < thread_2ch->responses_length)
 	  {
 	    offset = responses[start];
 	    if (offset != 0)
@@ -833,20 +781,31 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	fprintf(stderr, "1: ochusha_thread_2ch.c: length=%d\n",
 		buffer->length);
 #endif
-	while (rest_of_data > 0 && rest_of_responses > 0
-	       && i < MAX_RESPONSE	/* XXX: ܤꤹ */
+	while (rest_of_data > 0
+	       && (rest_of_responses > 0 || rest_of_responses == -1)
 	       && (buffer_fixed || interval-- > 0)
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
 	    OchushaBBSResponse response = { NULL, NULL, NULL, NULL };
 	    char *title_pos;
 
+	    if (i >= thread_2ch->responses_length)
+	      {
+		guint new_len = thread_2ch->responses_length * 2;
+		thread_2ch->responses = G_REALLOC(thread_2ch->responses,
+						  new_len);
+		memset(thread_2ch->responses + thread_2ch->responses_length, 0,
+		       (new_len - thread_2ch->responses_length)
+		       * sizeof(guint));
+		thread_2ch->responses_length = new_len;
+		responses = thread_2ch->responses;
+	      }
 #if DEBUG_DAT_MOST
 	    fprintf(stderr, "i=%d, offset=%d, rest_of_data=%d, cur_pos=%p\n",
 		    i, offset, rest_of_data, cur_pos);
 #endif
 
-	    title_pos = extract_response(cur_pos, eol_pos, &response);
+	    title_pos = extract_2ch_response(cur_pos, eol_pos, &response);
 
 #if DEBUG_DAT_MOST
 	    fprintf(stderr, "title_pos=%p\n", title_pos);
@@ -872,13 +831,21 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 		    if (each_response_cb != NULL)
 		      result = (*each_response_cb)(thread, i, &response,
 						   callback_data);
+		    if (i > 1000
+			&& !(thread->flags & OCHUSHA_BBS_THREAD_STOPPED)
+			&& response.date_id != NULL
+			&& strcmp(response.date_id, "Over 1000 Thread") == 0)
+		      {
+			thread->flags |= OCHUSHA_BBS_THREAD_STOPPED;
+		      }
 		  }
 		else
 		  {
 		    if (broken_response_cb != NULL)
 		      result = (*broken_response_cb)(thread, i, callback_data);
 		  }
-		rest_of_responses--;
+		if (rest_of_responses != -1)
+		  rest_of_responses--;
 		if (!result)
 		  break;
 	      }
@@ -893,10 +860,11 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	      {
 		char *line;
 		fprintf(stderr, "response #%d: Couldn't analyze DAT file...ignoring parts.\n", i);
-		line = G_STRNDUP(buffer_top + responses[i], offset - responses[i]);
+		line = G_STRNDUP(buffer_top + responses[i -1],
+				 offset - responses[i - 1]);
 		fprintf(stderr, "failed: \"%s\"\n", line);
 		G_FREE(line);
-		fprintf(stderr, "offset=%d\n", responses[i]);
+		fprintf(stderr, "offset=%d\n", responses[i - 1]);
 	      }
 #endif
 
@@ -1017,39 +985,6 @@ ochusha_thread_2ch_get_url_for_response(OchushaBBSThread *thread, int from,
 	break;
       }
 
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      {
-	if (from >= to)
-	  to = from;
-	snprintf(url, PATH_MAX, "http://%s/bbs/read.pl?BBS=%s&KEY=%s&START=%d&END=%d&NOFIRST=TRUE",
-		 ochusha_bulletin_board_get_server(thread->board),
-		 ochusha_bulletin_board_get_id(thread->board),
-		 thread->id, from, to);
-	break;
-      }
-
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      {
-	if (from >= to)
-	  to = from;
-	snprintf(url, PATH_MAX, "http://%s%sbbs/read.cgi?BBS=%s&KEY=%s&START=%d&END=%d&NOFIRST=TRUE",
-		 ochusha_bulletin_board_get_server(thread->board),
-		 ochusha_bulletin_board_get_base_path(thread->board),
-		 ochusha_bulletin_board_get_id(thread->board),
-		 thread->id, from, to);
-	break;
-      }
-
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      {
-	if (from >= to)
-	  to = from;
-	snprintf(url, PATH_MAX, "./read.cgi?bbs=%s&key=%s&st=%d&to=%d&nofirst=true",
-		 ochusha_bulletin_board_get_id(thread->board),
-		 thread->id, from, to);
-	break;
-      }
-
     default:	/* ̤ݡ */
       return NULL;
     }
@@ -1070,33 +1005,10 @@ ochusha_thread_2ch_get_url_to_post(OchushaBBSThread *thread)
   if (thread_2ch->post_url != NULL)
     return thread_2ch->post_url;
 
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-      snprintf(url, PATH_MAX, "%sl10",
-	       ochusha_thread_2ch_get_base_url(thread_2ch));
-      break;
-
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-    case OCHUSHA_BBS_TYPE_JBBS_SHITARABA:
-      snprintf(url, PATH_MAX, "%s?BBS=%s&KEY=%s&LAST=10",
-	       ochusha_board_2ch_get_read_cgi_url(OCHUSHA_BOARD_2CH(thread->board)),
-	       ochusha_bulletin_board_get_id(thread->board),
-	       thread->id);
-      break;
-
-    case OCHUSHA_BBS_TYPE_MITINOKU:
-      snprintf(url, PATH_MAX, "%s?bbs=%s&key=%s&ls=10",
-	       ochusha_board_2ch_get_read_cgi_url(OCHUSHA_BOARD_2CH(thread->board)),
-	       ochusha_bulletin_board_get_id(thread->board),
-	       thread->id);
-      break;
-
-    default:	/* ̤ݡ */
-      break;
-    }
-
+  snprintf(url, PATH_MAX, "%sl10",
+	   ochusha_thread_2ch_get_base_url(thread));
   thread_2ch->post_url = G_STRDUP(url);
+
   return thread_2ch->post_url;
 }
 
@@ -1119,15 +1031,7 @@ ochusha_thread_2ch_post_supported(OchushaBBSThread *thread)
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), FALSE);
   g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(thread->board), FALSE);
 
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      return TRUE;
-
-    default:
-      return FALSE;
-    }
+  return TRUE;
 }
 
 
@@ -1137,6 +1041,7 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
 				 const OchushaBBSResponse *response)
 {
   OchushaThread2ch *thread_2ch;
+  OchushaThread2chClass *thread_2ch_class;
   OchushaBoard2ch *board_2ch;
   char *tmp_string = NULL;
   char *from = NULL;
@@ -1155,16 +1060,6 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   g_return_val_if_fail(response->mailto != NULL, FALSE);
   g_return_val_if_fail(response->content != NULL, FALSE);
 
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      break;
-
-    default:
-      return FALSE;	/* ̤ݡ */
-    }
-
   converter = iconv_open(ochusha_bulletin_board_get_response_character_encoding(thread->board), "UTF-8");
   g_return_val_if_fail(converter != (iconv_t)-1, FALSE);
 
@@ -1206,20 +1101,17 @@ ochusha_thread_2ch_post_response(OchushaBBSThread *thread,
   if (time == -1)
     goto error_exit;
 
-  /* XXX: ϡɥǥ󥰤㤦ΤϤäʤġġ*/
-  switch (thread->board->bbs_type)
-    {
-    case OCHUSHA_BBS_TYPE_2CH:
-      query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&time=%ld", from, mail, message, bbs, key, time);
-      break;
+  thread_2ch_class = OCHUSHA_THREAD_2CH_GET_CLASS(thread_2ch);
+  if (thread_2ch_class->make_post_response_message)
+    query = (*thread_2ch_class->make_post_response_message)(thread_2ch,
+							    from, mail,
+							    message, bbs,
+							    key, time);
+  else
+    query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&FROM=%s&mail=%s&MESSAGE=%s&bbs=%s&key=%s&time=%ld", from, mail, message, bbs, key, time);
 
-    case OCHUSHA_BBS_TYPE_MACHIBBS:
-      query = g_strdup_printf("submit=%%8F%%91%%82%%AB%%8D%%9E%%82%%DE&NAME=%s&MAIL=%s&MESSAGE=%s&BBS=%s&KEY=%s&TIME=%ld", from, mail, message, bbs, key, time);
-      break;
-
-    default:
-      goto error_exit;
-    }
+  if (query == NULL)
+    goto error_exit;
 
   post_result = ochusha_utils_2ch_try_post(broker, thread->board, query);
   if (post_result == OCHUSHA_UTILS_2CH_POST_NO_COOKIE)
